/*
 * Sentry Video System
 *
 * $Id: $
 */

/*
 * connect_host.c
 */

#include <stdio.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>

#include "sentry.h"

extern int debug;
extern char progname[];

extern int ctrlfd;
extern int datafd;
extern char connection_host[];


int
bind_socket(int sockfd, int port)
{
  struct sockaddr_in my_addr;
  int flag = 1;

  if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, 
		 (char *)&flag, sizeof(flag)) < 0) {
    fprintf(stderr, "%s: cannot setsockopt REUSEADDR\n", progname);
    return -1;
  }

  bzero((char *)&my_addr, sizeof(my_addr));
  my_addr.sin_family = AF_INET;
  my_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  my_addr.sin_port = htons(port);

  if (bind(sockfd, (struct sockaddr *)&my_addr, sizeof(my_addr)) < 0) {
    fprintf(stderr, "%s: cannot bind port %d.\n", progname, port);
    return -1;
  }

  return 0;
}

int
build_tcp_port(int port, struct sockaddr_in *serv)
{
  int sockfd;

  if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    fprintf(stderr, "%s: cannot create new TCP socket.\n", progname);
    return -1;
  }
  if (bind_socket(sockfd, port) < 0) {
    fprintf(stderr, "%s: please wait several seconds.\n", progname);
    close (sockfd);
    return -1;
  }
  if (connect(sockfd, (struct sockaddr *)serv, sizeof(struct sockaddr)) < 0) {
    fprintf(stderr, "%s: cannot connect to server [%s/%d].\n",
	    progname, inet_ntoa(serv->sin_addr), ntohs(serv->sin_port));
    close(sockfd);
    return -1;
  }

  fprintf(stderr, "\nConnected to [%s/%d] (sockfd=%d)\n",
	  inet_ntoa(serv->sin_addr), ntohs(serv->sin_port), sockfd);
  return sockfd;
}

int build_udp_port(int port, struct sockaddr_in *serv)
{
  int sockfd;
  int flag;
  int flaglen = sizeof(flag);

  if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    fprintf(stderr, "%s: cannot create new UDP socket.\n", progname);
    return -1;
  }

  flag = K(32);
  flag = K(64);
  flag = K(128);
  flag = K(256);
  if (setsockopt(sockfd, SOL_SOCKET, SO_RCVBUF,
		 (char *)&flag, flaglen) < 0) {
    fprintf(stderr, "%s: cannot setsockopt RCVBUF size %d.\n", 
	    progname, flag);
  }
  if (debug) {
    if (getsockopt(sockfd, SOL_SOCKET, SO_RCVBUF,
		   (char *)&flag, &flaglen) < 0) {
      fprintf(stderr, "%s: cannot getsockopt RCVBUF size.\n", progname);
      return -1;
    } else {
      fprintf(stderr, "%s: RCVBUF size = %d\n", progname, flag);
    }
  }

  if (debug)
    fprintf(stderr, "build_udp_port: sockfd=%d/ port=%d\n", sockfd, port);

  if (bind_socket(sockfd, port) < 0) {
    fprintf(stderr, "%s: please wait several seconds.\n", progname);
    close(sockfd);
    return -1;
  }

  if (debug)
    fprintf(stderr, "new udp socket %d [port=%d]\n", sockfd, port);
  
  return sockfd;
}

u_long
get_addr(char *hostname)
{
  u_long addr;
  struct hostent *hostptr;

  if (debug) 
    fprintf(stderr, "get_addr: hostname=#%s#\n", hostname);

  addr = htonl(inet_addr(hostname));
  if (addr == 0) { /* no address */
    print_message("no address");
    return -1;
  } else if (addr == (u_int)-1) {
    if ((hostptr = gethostbyname(hostname)) == NULL) {
      print_message("gethostbyname error.");
      return -1;
    }
    if (hostptr->h_addrtype != AF_INET) {
      print_message("gethostbyname h_addrtype error.");
      return -1;
    }
    bcopy(hostptr->h_addr, (char *)&addr, hostptr->h_length);
  }

  if (debug)
    fprintf(stderr, "get_addr: addr=%x\n", addr);

  return addr;
}


/*
 * connect_host() - establish tcp connection to server.
 */
int
connect_host(char *hostname, int port)
{
  int sockfd;
  struct sockaddr_in serv_addr;

  if (debug)
    fprintf(stderr, "connect_host: hostname=#%s#/ port=%d\n", 
	    hostname, port);

  if ((serv_addr.sin_addr.s_addr = get_addr(hostname)) < 0)
    return -1;
  serv_addr.sin_family = AF_INET;
  serv_addr.sin_port = htons(port);

  /*
   * create cntrl socket
   */
  if ((sockfd = build_tcp_port(0, &serv_addr)) < 0) {
    fprintf (stderr, "connect_host: cannot build tcp port.\n");
    return -1;
  }

  return sockfd;
}


void
disconnect_host(void)
{

  if (debug)
    fprintf(stderr, "disconnect_host: ctrlfd=%d/ datafd=%d\n",
	    ctrlfd, datafd);

  if (datafd >= 0) {
    shutdown(datafd, 2);
    close(datafd);
    datafd = -1;
  }

  if (ctrlfd >= 0) {
    request_call(SENTRY_CLOSE);

    sleep(1);
    shutdown(ctrlfd, 2);
    close(ctrlfd);
    ctrlfd = -1;
  }
}


/*
 * build_dataport() - build data socket
 */
int 
build_dataport(char *hostname, int port)
{
  int sockfd;
  struct sockaddr_in serv_addr;

  if (debug)
    fprintf(stderr, "build_dataport: hostname=#%s#/ port=%d\n",
	    hostname, port);

  if ((serv_addr.sin_addr.s_addr = get_addr(hostname)) < 0)
    return -1;
  serv_addr.sin_family = AF_INET;
  serv_addr.sin_port = htons(0);

  if ((sockfd = build_udp_port(port, &serv_addr)) < 0)
    return -1;

  if (debug)
    fprintf(stderr, "built datafd=%d\n", sockfd);

  return sockfd;
}


/*
 * do_up() - connect to sentry server
 */
int
do_up(char *hostname, int server_port)
{
  char mesg[128];

  if (hostname == NULL || hostname[0] == '\0') {
    sprintf(mesg, "empty hostname");
    print_message(mesg);
    return -1;
  } else {
    strcpy(connection_host, hostname);

    /*
     * check existance of hostname
     */
    if (get_addr(connection_host) < 0) {
      sprintf(mesg, "ignore hostname \"%s\"", connection_host);
      return -1;
    }

    if (ctrlfd >= 0) {
      disconnect_host();
    }

    /*
     * crete ctrl socket and data socket
     */
    if ((ctrlfd = connect_host(connection_host, server_port)) < 0 ||
	(datafd = build_dataport(connection_host, server_port-1)) < 0) {
      

      if (debug)
	fprintf(stderr, "%s: ctrlfd=%d/ datafd=%d\n", 
		progname, ctrlfd, datafd);

      sprintf(mesg, "Unconnection host ... %s", connection_host);
      print_message(mesg);

      disconnect_host();
      
      return 1;
    }

  }
  return 0;
}

