/*
 * Sentry Video System
 *
 * $Id: $
 */

/*
 * get_data.c - get data (image header and body) from dgram socket
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/select.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>

#include <X11/Intrinsic.h>

#include "sentry.h"

extern char progname[];

char    *buff = NULL;
char    *conv_buff = NULL;
#if 0
char    *out_buff = NULL;
#endif

/*
 * get_data() - callback of getting data from server
 */
int
get_data(int datafd, void *(image_proc()))
{
  struct sockaddr_in from;
  int fromlen = sizeof(from);
  char packet[MAX_DGRAM_LEN];
  int nread;
  static int last_size;
  int image_size, image_width, image_height;
  static int recv_count = 0;
  char mesg[128];
  
  int recv_size = 0;

  /*
  fd_set InputFds;
  extern fd_set InputFdMask;
  */

  extern int udp_reassemble();
  extern char *data_expand();

  register sentry_image_header *hdr;

  /*
   * receive packets
   */
  while (1) {
    memset(packet, 0, MAX_DGRAM_LEN);

    nread = recvfrom(datafd, packet, sizeof(packet), 0, 
		   (struct sockaddr *)&from, &fromlen);
    if (debug >= 100)
      fprintf(stderr, "get_data: %d bytes received from [datafd=%d].\n",
	      nread, datafd);

    /*
     * header check
     */
    if (nread < 0)
      goto fail;
    if ((nread -= SENTRY_HEADER_LEN) < 0) {
      fprintf(stderr, "%s: header length is too short.", progname);
      return;
    }

    hdr = (sentry_image_header *)packet;
    if (debug >= 100) {
      fprintf(stderr, "*** sentry image header ***\n");
      fprintf(stderr, "   hd_ver = %d\n", hdr->hd_ver);
      fprintf(stderr, "   hd_ts  = %08x\n", hdr->hd_ts);
      fprintf(stderr, "   hd_off = %8d\n", hdr->hd_off);
      fprintf(stderr, "   size   = %d\n", hdr->size);
      fprintf(stderr, "   width  = %d\n", hdr->width);
      fprintf(stderr, "   height = %d\n", hdr->height);
      fprintf(stderr, "   format = %d\n", hdr->format);
      fprintf(stderr, "   encode = %d\n", hdr->encode);
    }

    if (hdr->hd_ver != SENTRY_VERSION) {
      fprintf(stderr, "%s: version error.", progname);
      return;
    }
  
    /*
     * reassemble udp packets
     */
    recv_count ++;
    if ((recv_size = 
	 udp_reassemble(hdr, &packet[SENTRY_HEADER_LEN], nread, &buff)) > 0) {
#if 0
      fprintf(stderr, "get_data: okay. get image size %8ld\n", 
	      (long)recv_size);
#endif
      break;
    } else {
#if 0
      fprintf(stderr, "get_data: more. get image size %8ld\n",
	      (long)recv_size);
#endif
      /*
      return;
      */
    }

    /*
    return;
    */
  } /* end of while-loop */
    
  recv_count = 0;
  
  /*
   * get packet timestamp
   */
  /* no-op */

  /*
   * packet reassemble completed
   */
  last_size = recv_size;
  image_width = hdr->width;
  image_height = hdr->height;

  if (debug >= 100)
    fprintf(stderr, "get_data: hdr->format=%d\n", hdr->format);

  if (hdr->format == MONOCHROME) {
    /*
     * monochrome
     */
    fprintf(stderr, "Sorry, monochrome image is not supproted. exit.\n");
    exit (1);
  } else if (hdr->format == FULL_COLOR) {
    /* 
     * full color
     */
    conv_buff = buff;
  }


#if 0
  /*
   * get frame type
   */
  if (hdr->encode == ENCODE_JPEG) {
    if (decode_jpeg(conv_buff, image_width, image_height, &out_buff) < 0) 
      return 0;
  } else {
    out_buff = conv_buff;
  }
#endif

  image_proc(last_size, image_width, image_height, hdr->encode, conv_buff);
  goto out;

fail:
  fprintf(stderr, "recvfrom() error. server is down ?\n");
  fflush(stderr);

  free(buff);
  return -1;

out:
  return 0;
}

  
    
