/*
 * Sentry Video System
 *
 * $Id: $
 */
#include <stdio.h>
#include <sys/types.h>
#include <errno.h>

#include <dmedia/cl.h>

extern int debug;
extern int errno;

static int jpeg_system_initted = 0;

static CLhandle decompressorHdl;

void *inner_buff = NULL;


int
init_jpeg_system(int x, int y)
{
  int chk;

  jpeg_system_initted = 1;
  
  chk = clOpenDecompressor(CL_JPEG_COSMO, &decompressorHdl);
  if (chk != SUCCESS) {
    fprintf(stderr, "init_jpeg_system: no COSMO system.\n");

    chk = clOpenDecompressor(CL_JPEG_SOFTWARE, &decompressorHdl);
    if (chk != SUCCESS) {
      fprintf(stderr, "init_jpeg_system: no JPEG library.\n");
      return -1;
    } else
      fprintf(stderr, "init_jpeg_system: use jpeg software decoding.\n");
  } else {
    fprintf(stderr, "init_jpeg_system: have COSMO system.\n");
  }

  clSetParam(decompressorHdl, CL_IMAGE_WIDTH, x);
  clSetParam(decompressorHdl, CL_IMAGE_HEIGHT, y);
  clSetParam(decompressorHdl, CL_FORMAT, CL_FORMAT_XBGR);


  if (inner_buff != NULL) {
    free(inner_buff);
    inner_buff = NULL;
  }
  
  if ((inner_buff = malloc(x*y*5)) == NULL) {
    fprintf(stderr, "init_jpeg_system: no memory (%d)\n", errno);
    return -1;
  }

  return 0;
}
    

int
decode_jpeg(char *cur, int csize, int x, int y, char **new)
{
  if (jpeg_system_initted == 0) 
    if (init_jpeg_system(x,y) < 0)
      return -1;

  if (clDecompress(decompressorHdl, 1, csize, (void *)cur, inner_buff) < 0)
    fprintf(stderr, "decode_jpeg: clDecompress() error.\n");

  *new = inner_buff;

  return 0;
}

