/*
 * Sentry Video System
 *
 * $Id: $
 */

/*
 * put_image.c - output image procedures
 */
#include <stdio.h>
#include <stdlib.h>

#include <X11/Intrinsic.h>
#include <X11/Xlib.h>
#include <Xm/XmAll.h>

#include "sentry.h"

static XImage       *ximage = NULL;

extern int debug;

extern XtAppContext app;
extern Display      *display;
extern Window       window;
extern GC           gc;
extern Visual       *visual;
extern int          depth;
extern char         *connection_host;

int client_zoom = 1;

extern void print_message();

extern Widget image_shell;

extern int frame_per_sec;



/*
 * put_image
 */
void
put_image(int width, int height, char *data)
{
  char mesg[128];
  static int last_width, last_height;

  if (debug >= 10)
    fprintf(stderr, "put_image: data=%d/ ximage=%d [width#last=%d/cur=%d/ height#last=%d/cur=%d] client_zoom=%d\n",
	    data, ximage, last_width, width*client_zoom,
	    last_height, height*client_zoom, client_zoom);

  if (ximage == NULL ||
      last_width != width*client_zoom || 
      last_height != height*client_zoom) {
    printf("put_image: (%d x %d)\n",
	   width*client_zoom, height*client_zoom);

    if (ximage != NULL)
      XDestroyImage(ximage);

    ximage = XCreateImage(display, visual, depth, ZPixmap, 0, 0,
			  width*client_zoom, height*client_zoom, 
			  BitmapPad(display), 0);
    if (debug)
      fprintf(stderr, "created ximage [addr=%d]\n", ximage);

    if (ximage == NULL) {
      fprintf(stderr, "XCreateImage() failed.\n");
      print_message("XCreateImage() failed.");
      return;
    }
  }
  ximage->data = data;

  last_width = width*client_zoom;
  last_height = height*client_zoom;

  XPutImage(display, window, gc, ximage, 
	    0, 0, 0, 0, last_width, last_height);

  frame_per_sec++;

  if (debug >= 10) {
    fprintf(stderr, "put_image: XPutImage done.\n");
    /*
    save_image(width, height, data);
    */
  }

  XFlush(display);
}


char *out_buff = NULL;

void
show_image(int size, int width, int height, int encode_type, char *data)
{
  if (encode_type == ENCODE_JPEG) {
    if (decode_jpeg(data, size, width, height, &out_buff) < 0)
      return;
  } else {
    out_buff = data;
  }

  put_image(width, height, out_buff);
}
