/*
 * Sentry Video System
 *
 * $Id: $
 */

/*
 * recorder.c - recorder
 */

#include <stdio.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/stropts.h>

#include <fcntl.h>
#include <errno.h>

#include "sentry.h"

#ifndef WAIT_TIME
#define WAIT_TIME       (3)
#endif

int debug;

extern char sentry_savepathbase[];

char progname[] = "recorder";

int     ctrlfd;
int     datafd;
fd_set  InputFdMask, InputFds;

char    connection_host[128];
int     chk;
static int count;

int     wait_time = WAIT_TIME;

/***************************************************************
 *
 * default configuration
 *
 ***************************************************************/
int     client_type = SENTRY_UNICAST_OPEN;
#if 1
int     server_size = SIZE_212x160;
int     server_zoom = SIZE_212x160;
#else
int     server_size = SIZE_160x120;
int     server_zoom = SIZE_160x120;
#endif
int     server_color = FULL_COLOR;
int     server_encode = ENCODE_JPEG;
int     server_rate = 1000;
int     server_port = SENTRY_CTRL_PORT;
#define RATEVIEW_INTERVAL 1500
int     rateview_interval = RATEVIEW_INTERVAL;

/*
 * signal handler 
 */
void
complete(int sig)
{
  if (debug)
    fprintf(stderr, "complete: sig=%d is trapped.\n", sig);

  disconnect_host();
  exit (0);
}

volatile sig_atomic_t server_down = 0;

void
server_down_trap(void)
{
  server_down = 1;

  fprintf(stderr, "%s: server is down (%d)\n", progname, server_down);

#ifdef SOLARIS
  signal(SIGPOLL, SIG_IGN);
#endif

  return;
}

void
retry_connect(int sig)
{
  if (debug)
    fprintf(stderr, "retry_connect: sig=%d\n", sig);

  if (ctrlfd < 0) {
    fprintf(stderr, "retry_connect: connect to %s\n", connection_host);
    if (do_up(connection_host)) 
      fprintf(stderr, "retry_connect: connections fail !\n");
  } else {
    fprintf(stderr, "retry_connect: already connected to %s\n",
	    connection_host);
  }
}

int
set_iotrap(int fd)
{
  int flg;
  int val;

  flg = getpid();

#ifdef SOLARIS
  if ((val = ioctl(fd, I_SETSIG, S_INPUT)) < 0) {
    fprintf(stderr, "fail ioctl() with I_SETSIG (errno=%d)\n", errno);
    perror("set_iotrap: ioctl: I_SETSIG");
  } else {
    if (debug)
      fprintf(stderr, "success ioctl() with I_SETSIG (val %d)\n", val);
  }
#endif

  if ((flg = fcntl(fd, F_GETFL, 0)) < 0) {
    fprintf(stderr, "fail fcntrl() with F_GETFL (errno=%d)\n", errno);
  } else {
    if (debug)
      fprintf(stderr, "success fcntl() with F_GETFL (val %d)\n", val);
  }
#ifdef FASYNC
  flg |= FASYNC;
#else
#ifdef O_ASYNC
  flg |= O_ASYNC;
#else
  flg = 0;
#endif
#endif
  if (flg && fcntl(fd, F_SETFL, flg) < 0) {
    fprintf(stderr, "fail fcntl() with F_SETFL (errno %d)\n", errno);
  } else {
    if (debug)
      fprintf(stderr, "success fcntl() with F_SETFL (val %d)\n", flg);
  }
}

void
usage(void)
{
  printf("Usage: %s [options] hostname\n\n", progname);
  printf("option:\n");
  printf("       -h       help\n");
  printf("       -s type  frame size\n");
  printf("                    1 640x480\n");    
  printf("                    2 320x240\n");
  printf("                    2 212x160 (deafault)\n");
  printf("                    2 160x120\n");
  printf("       -r num   frame rate (frame per 100 sec)\n");
  printf("       -d dir   strage directory (default=%s)\n", 
	 sentry_savepathbase);
  printf("\n");

  exit (1);
}

void
put_message(char *mesg)
{
  printf("##%s\n", mesg);
}


/*
 * recorder main routine
 */
main(int argc, char **argv)
{
  int c;
  extern char *optarg;
  void init_recorder(), mainloop();

#ifdef DEBUG
  debug = 0;
#endif

  signal(SIGQUIT, complete);
  signal(SIGTERM, complete);
  signal(SIGINT, complete);

  while ((c = getopt(argc, argv, "hDd:s:r:")) != -1) {
    switch(c) {
    case 'h':
      usage();
      exit (1);
    case 'D':
      debug = 1;
      break;
    case 'd':
      if (strlen(optarg) < SENTRY_DIRLEN) {
	strcpy(sentry_savepathbase, optarg);
      } else {
	fprintf(stderr, "specified path is too long '%s'\n", optarg);
	exit (2);
      }
      break;
    case 's':
      server_size = atoi(optarg);
      server_zoom = atoi(optarg);
      break;
    case 'r':
      server_rate = atoi(optarg);
      break;
    }
  }

  /*
   * Initialize variable
   */
  ctrlfd = datafd = -1;

  /*
   * set server name
   */
  if (argc > 1) {
    strcpy(connection_host, argv[argc-1]);
  } else {
    usage();
  }

  /*
   * initialize recorder
   */
  init_recorder();

  /*
   * recorder mainloop
   */
  mainloop();

  /* NOT REACHED */
}

void
init_recorder()
{
  char mesg[128];


#if 1
  while(chk = do_up(connection_host, server_port)) {
    if (chk < 0) {
      fprintf(stderr, "%s: server not found.", progname);
      exit (1);
    }
    fprintf(stderr, "%s: server is not ready.", progname);
    sleep(wait_time);
  }
    
  server_down = 0;

#else    
start:
  if (chk = do_up(connection_host, server_port)) {
    if (chk < 0) {
      fprintf(stderr, "%s: server not found.", progname);
    } else {
      fprintf(stderr, "%s: server is not ready.", progname);

      sleep(wait_time);
      goto start;
    }
  } else {
    server_down = 0;
  }
#endif

  signal(SIGCONT, retry_connect);
#ifdef SOLARIS
  signal(SIGPOLL, server_down_trap);
#else
  signal(SIGIO, server_down_trap);
#endif
  signal(SIGURG, server_down_trap);

  set_iotrap(ctrlfd);
  init_lbuf();

  request_call(client_type);
}


void
mainloop(void)
{
  void init_recorder();
  extern void save_image();

  while(1) {
    if (server_down) {
      disconnect_host();

      fprintf(stderr, "%s: server is down. waiting...\n", progname);
      sleep(wait_time);
      init_recorder();
    }

    get_data(datafd, &save_image);
  }
}

void
set_timer(int interval, void *func)
{
  static struct itimerval view_timer;
  int int_sec, int_usec;

  int_sec = interval / 1000;
  int_usec = interval * 1000 % 1000000;

  fprintf(stderr, "#set_timer: int_sec=%d/ usec=%d\n", int_sec, int_usec);

  int_sec = (interval * 1000 - int_usec) / 1000000;

  view_timer.it_interval.tv_sec = int_sec;
  view_timer.it_interval.tv_usec = int_usec;
  view_timer.it_value.tv_sec = int_sec;
  view_timer.it_value.tv_usec = int_usec;

  signal(SIGALRM, func);
  setitimer(ITIMER_REAL, &view_timer, NULL);
}

void
print_message(char *mesg)
{
  fprintf(stderr, "%s\n", mesg);
  fflush(stderr);
}




  
