/*
 * Sentry Video System
 *
 * $Id: $
 */

/*
 * save_image.c - save image to file
 */
#include <stdio.h>
#include <sys/stat.h>
#include <sys/dir.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/errno.h>

#include "sentry.h"

#ifndef SENTRY_DIRLEN
#define SENTRY_DIRLEN   (256)
#endif

#ifndef SENTRY_SAVEPATHBASE
#define SENTRY_SAVEPATHBASE     "/var/tmp/sentry_save"
#endif

extern int debug;
extern int errno;

char sentry_savepathbase[SENTRY_DIRLEN] = SENTRY_SAVEPATHBASE;
int absolute_path = 0;

int p_sec=-1, p_min=-1, p_hour=-1;
int p_mday=-1, p_mon=-1, p_year=-1;

int
set_path_bydate(char *rpath, char *base, char *ext, int *c)
{
  time_t t;
  struct tm *tm;
  static first=1;
  char dir_date[SENTRY_DIRLEN],
    dir_hour[SENTRY_DIRLEN],
    dir_min[SENTRY_DIRLEN],
    dir_sec[SENTRY_DIRLEN];

  

  t = time(NULL);
  tm = localtime(&t);

  if (debug >= 100) {
    fprintf(stderr, 
	    "set_path_bydate: date and time %d%02d%02d %02d:%02d:%02d %02d\n",
	    tm->tm_year, tm->tm_mon, tm->tm_mday,
	    tm->tm_hour, tm->tm_min, tm->tm_sec, *c);
    fprintf(stderr, 
	    "set_path_bydate: date and time(prev): %d%02d%02d %02d:%02d:%02d %02d\n",
	    p_year, p_mon, p_mday, p_hour, p_min, p_sec, *c);
  }

  if (p_year == tm->tm_year && p_mon == tm->tm_mon && p_mday == tm->tm_mday) {
    if (p_hour == tm->tm_hour) {
      if (p_min == tm->tm_min) {
	if (p_sec == tm->tm_sec) {
	  goto set_frame;
	}
	*c = 0;
	goto set_sec;
      }
      *c = 0;
      goto set_min;
    }
    *c = 0;
    goto set_hour;
  }
  *c = 0;
  goto set_date;


set_date:;
  if (first) {
    if (debug >= 100)
      fprintf(stderr, "get_path_bydate: base_dir %s\n", sentry_savepathbase);
    umask(022);

    if (mkdir(sentry_savepathbase, 0755) < 0) {
      if (errno != EEXIST) {
	perror("set_path_bydate: mkdir base_dir");
	fprintf(stderr, "set_path_bydate: base_dir '%s' is not ready.\n",
		sentry_savepathbase);

      }
    }
    first = 0;
  }

  p_year = tm->tm_year;
  p_mon = tm->tm_mon;
  p_mday = tm->tm_mday;

  sprintf(dir_date, "%s/%d%02d%02d", sentry_savepathbase,
	  p_year, p_mon+1, p_mday);
  fprintf(stderr, "set_path_bydate: dir_date=#%s#\n", dir_date);

  if (mkdir(dir_date, 0755) < 0) {
    if (errno != EEXIST) 
      perror("set_path_bydate: mkdir dir_date");
  }


set_hour:;
  p_hour = tm->tm_hour;
  sprintf(dir_hour, "%s/%d%02d%02d/%02d", 
	  sentry_savepathbase, p_year, p_mon+1, p_mday, p_hour);
  fprintf(stderr, "set_path_bydate: dir_hour=#%s#\n", dir_hour);

  if (mkdir(dir_hour, 0755) < 0) {
    if (errno != EEXIST)
      perror("set_path_bydate: mkdir dir_hour");
  }


set_min:;
  p_min = tm->tm_min;
  sprintf(dir_min, "%s/%d%02d%02d/%02d/%02d", 
	  sentry_savepathbase, p_year, p_mon+1, p_mday, p_hour, p_min);
  fprintf(stderr, "set_path_bydate: dir_min=#%s#\n", dir_min);

  if (mkdir(dir_min, 0755) < 0) {
    if (errno != EEXIST)
      perror("set_path_bydate: mkdir dir_min");
  }
	  
  if (chdir(dir_min) < 0) {
    absolute_path = 1;
    perror("set_path_bydate: chdir dir_min");
  } else {
    fprintf(stderr, "set_path_bydate: chdir #%s#\n", dir_min);
    absolute_path = 0;
  }


set_sec:;
  p_sec = tm->tm_sec;

set_frame:;
  if (absolute_path) {
    sprintf(rpath, "%s/%d%02d%02d/%02d/%02d/%02d-%02d.%s",
	    sentry_savepathbase, 
	    p_year, p_mon+1, p_mday, p_hour, p_min, p_sec, *c, ext);
  } else {
    sprintf(rpath, "%02d-%02d.%s", p_sec, *c, ext);
  }

  (*c)++;

  if (debug >= 100) 
    fprintf(stderr, "set_path_bydate:       -> %s\n", rpath);
}


/*
 * save image to file
 */
static char sfname[SENTRY_DIRLEN];

void
save_jpeg(int width, int height, char *data)
{
  void save_thru();

  fprintf(stderr, "save_jpeg: not yet implemented. save_thru...\n");
  save_thru(width, height, data);
}
      
void
save_thru(int width, int height, char *data)
{
  static int c = 0;
  char wfname[SENTRY_DIRLEN];
  int nw;
  int outfd;

  set_path_bydate(sfname, sentry_savepathbase, "jpg", &c);
  sprintf(wfname, "%s/current.jpg", sentry_savepathbase);

  if ((outfd = open(wfname, (O_CREAT|O_WRONLY|O_TRUNC), 0644)) < 0) {
    perror("save_thru: open");
    return;
  }

  nw = write(outfd, data, width);
  if (nw >= 0) {
    if (debug)
      fprintf(stderr, "save_thru: nw=%d\n", nw);
  } else {
    perror("save_thru: write");
  }

  if (rename(wfname, sfname) < 0)
    perror("save_thru: rename");

}


void
save_image(int size, int width, int height, int encode_type, char *data)
{
  if (encode_type == ENCODE_JPEG) {
    save_thru(size, 0, data);
  } else if (encode_type == ENCODE_NONE) {
    save_jpeg(width, height, data);
  }
}


  





