/*
 * Sentry Video System
 *
 * $Id: $
 */

/*
 * sentry.c - sentry client main routine
 */
#include <stdio.h>
#include <sys/signal.h>

#include <X11/Intrinsic.h>
#include <Xm/XmAll.h>

#include "sentry.h"

int     debug;

char    progname[] = "sentry";

int     ctrlfd;
int     datafd;
fd_set  InputFdMask, InputFds;

Display     *display;
Window      window;
Visual      *visual;
int         depth;
GC          gc;
int         popup_flg = 0;

XtAppContext    app;
XtInputId       iid;
Widget          toplevel, error_dialog, image_shell;

char    connection_host[128];
int     chk;
int     message_widget_up;
static int count;

/***************************************************************
 *
 * default configuration
 *
 ***************************************************************/
int     client_type = SENTRY_UNICAST_OPEN;
#if 1
int     server_size = SIZE_212x160;
int     server_zoom = SIZE_212x160;
#else
int     server_size = SIZE_160x120;
int     server_zoom = SIZE_160x120;
#endif
int     server_color = FULL_COLOR;
int     server_encode = ENCODE_JPEG;
int     server_rate = 2000;
int     server_port = SENTRY_CTRL_PORT;
#define RATEVIEW_INTERVAL 1000
int     rateview_interval = RATEVIEW_INTERVAL;


/*
 * signal handler
 */
void
complete(int sig)
{
  if (debug)
    fprintf(stderr, "complete: sig=%d is trapped.\n", sig);

  disconnect_host();
  exit (0);
}

/*
 * Connect Callback routine
 */
void 
ConnectCB(Widget w, XtPointer client_data, XtPointer call_data)
{
  Widget text = (Widget)client_data;
  char *serv_name;
  char mesg[128];
  int do_up();
  String tmp;
  void make_connect_dialog(), print_message(), put_message();
  void GetdataCB(), rate_view();

  serv_name = XmTextFieldGetString(text);
  if (connection_host[0] != '\0' && 
      (!strcmp(connection_host, serv_name))) {
    return;
  } 

  if (chk = do_up(serv_name, server_port)) {
    if (chk < 0) {
      sprintf(mesg, "%s: server not found.", progname);
    } else {
      sprintf(mesg, "%s: server is not ready.", progname);
      print_message(mesg);
    }
    if (popup_flg) {
      if (debug)
	fprintf(stderr, "clear window.\n");
      XtRemoveInput(iid);
      XtUnmanageChild(image_shell);
      popup_flg = 0;
      /*
      XClearWindow(display, window);
      XStoreName(display, window, "none");
      */
    }

#if UNDER_CONSTRUCT
    XtVaSetValues(serv_text, XmNvalue, NULL, NULL);
#endif
    /*
    XtUnmanageChild(XtParent(w));
    make_connect_dialog();
    */
    if (debug)
      fprintf(stderr, "ConnectCB: error return.\n");
    return;
  }
  sprintf(mesg, "Connected to \"%s\"", serv_name);
  put_message(mesg);
  XtFree(serv_name);

  /*
   * set popup window name
   */
  if (make_image_window() < 0) {
    fprintf(stderr, "cannot create image window.\n");
    exit (1);
  }
  XStoreName(display, window, connection_host);

  if (debug) {
    fprintf(stderr, "connect %s successfully.\n", connection_host);
    fprintf(stderr, "#%d: ctrlfd=%d/ datafd=%d\n", 
	    ++count, ctrlfd, datafd);
  }

  /*
   * set X input handler (callback routine)
   */
  FD_SET(datafd, &InputFdMask);

  request_call(client_type);
  iid = XtAppAddInput(app, datafd, (caddr_t)XtInputReadMask, 
		      GetdataCB, NULL);
  XtAppAddTimeOut(app, rateview_interval, rate_view, NULL);
}

void
make_connect_dialog(void)
{
  Widget serv_panel, serv_label, serv_text, serv_connect;
  void ConnectCB();

  serv_panel = XmCreateRowColumn(toplevel, "serv_panel", NULL, 0);

  serv_label = XmCreateLabel(serv_panel, "serv_label", NULL, 0);
  XtManageChild(serv_label);

  serv_text = XmCreateTextField(serv_panel, "serv_text", NULL, 0);
  XtManageChild(serv_text);  

  serv_connect = XmCreatePushButton(serv_panel, "connect_button", NULL, 0);
  XtManageChild(serv_connect);

  XtAddCallback(serv_connect, XmNactivateCallback, ConnectCB, serv_text);

  XtVaSetValues(toplevel, XmNinitialFocus, serv_panel, NULL);
  XtVaSetValues(serv_panel, XmNinitialFocus, serv_text, NULL);
  XtManageChild(serv_panel);
}




main(int argc, char **argv)
{
  int i;
  Arg arglist[20];
  Widget serv_panel, serv_label, serv_text, serv_connect;
  void ConnectCB(), make_message_widget();

#ifdef DEBUG
  debug = 0;
#endif

  signal(SIGQUIT, complete);
  signal(SIGTERM, complete);
  signal(SIGINT, complete);

  /*
   * Initialize toolkit and parse command line options
   */
  XtSetLanguageProc(NULL, NULL, NULL);

  toplevel = XtAppInitialize(&app, "Sentry", NULL, 0, &argc, argv,
			     NULL, NULL, 0);

  /*
   * Initialize variable
   */
  connection_host[0] = '\0';
  ctrlfd = datafd = -1;

  /*
   * error message widget
   */
  make_message_widget();

  /*
   * server connection dialog
   */
  make_connect_dialog();
	    
  XtRealizeWidget(toplevel);
  XtAppMainLoop(app);
}


void
make_message_widget(void)
{
  Arg args[10];
  int ac = 0;
  Widget widget;

  ac = 0;
  XtSetArg(args[ac], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ac++;
  error_dialog = XmCreateErrorDialog(toplevel, "error_dialog", args, ac);
  
  widget = XmMessageBoxGetChild(error_dialog, XmDIALOG_CANCEL_BUTTON);
  widget = XmMessageBoxGetChild(error_dialog, XmDIALOG_HELP_BUTTON);

  message_widget_up = 1;
}

void
popup_message(char *message)
{
  XmString xms_message;

  if (debug)
    fprintf(stderr, "popup_message: mesg=\"%s\"\n", message);

  xms_message = XmStringCreateSimple(message);
  XtVaSetValues(error_dialog, XmNmessageString, xms_message, NULL);
  XtManageChild(error_dialog);
}


void
put_message(char *mesg)
{
  printf("##%s\n", mesg);
}

int
make_image_window(void)
{
  int screen;
  char mesg[128];
  XVisualInfo vinfo;
  Widget image_base;
  Widget image_sizelabel, image_canvas;
  XmString tmp;
  Arg args[10];
  int ac = 0;
  Colormap colormap;
  XSetWindowAttributes attributes;
  u_long valuemask;
  XGCValues values;
  void print_message();

  if (debug)
    fprintf(stderr, "popup_flg=%d\n", popup_flg);
  if (popup_flg == 0) {
    sprintf(mesg, "%s connection ... ", connection_host);
    put_message(mesg);

    display = XtDisplay(toplevel);
    screen = DefaultScreen(display);

    if (XMatchVisualInfo(display, screen, 24, TrueColor, &vinfo) == 0 &&
	XMatchVisualInfo(display, screen, 32, TrueColor, &vinfo) == 0) {
      put_message("Popup PseudoColor Window");
      print_message("XMatchVisualInfo() failed.");
      return -1;
    }
    visual = vinfo.visual;
    depth = vinfo.depth;

    printf("visual.visualid=%x/ depth=%d\n", visual->visualid, depth);

    /*
    image_shell = XmCreateFormDialog(toplevel, "ImageShell", NULL, 0);
    */
    /*
    image_base = XmCreateDialogShell(toplevel, "ImageBase", NULL, 0);
    image_base = XmCreateMainWindow(toplevel, "ImageBase", NULL, 0);
    */
    image_base = XtVaAppCreateShell("Sentry", "ImageBase",
				    topLevelShellWidgetClass,
				    XtDisplay(toplevel), NULL);
    image_shell = XmCreateForm(image_base, "ImageShell", NULL, 0);
    XtManageChild(image_shell);

    tmp = XmStringCreateSimple("Size 000x000");
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, tmp); ac++;
    image_sizelabel = XmCreateLabel(image_shell, "ImageLabel", args, ac);
    XtManageChild(image_sizelabel);
    XmStringFree(tmp);

    /*
    if (debug)
      goto popup;
      */

    ac = 0;    
    XtSetArg(args[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
    XtSetArg(args[ac], XmNtopWidget, image_sizelabel); ac++;
    image_canvas = XmCreateDrawingArea(image_shell, "ImageCanvas", args, ac);
    XtManageChild(image_canvas);

    display = XtDisplay(image_canvas);
    screen = DefaultScreen(display);

    colormap = XCreateColormap(display, RootWindow(display, screen),
			       visual, AllocNone);
    attributes.colormap = colormap;
    attributes.background_pixel = BlackPixel(display, screen);
    attributes.border_pixel = BlackPixel(display, screen);
    valuemask = CWColormap | CWBackPixel | CWBorderPixel;
    
    window = XCreateWindow(display, RootWindow(display, screen),
			   0, 0, 212, 160, 0,
			   depth, InputOutput, visual, 
			   valuemask, &attributes);
    XMapWindow(display, window);
    XFlush(display);

    values.background = values.foreground = WhitePixel(display, screen);
    valuemask = GCForeground | GCBackground;
    gc = XCreateGC(display, window, valuemask, &values);

    /*
     * popup window 
     */
  popup:
    popup_flg = 1;
    XtRealizeWidget(image_base);
  }
    
  return 1;
}

void
GetdataCB(caddr_t client_data, int *fd, XtInputId *id)
{
  int datafd = *fd;
  extern int get_data();
  extern void show_image();
  
  if (get_data(datafd, &show_image) < 0) {
    XtRemoveInput(*id);
    disconnect_host();
  }
  return;
}

void
set_timer(int interval, XtTimerCallbackProc func)
{
  
  if (popup_flg) {
    XtAppAddTimeOut(app, interval, func, NULL);
  }
#if 0
  if (debug)
    fprintf(stderr, "set_timer: *func=%d\n", func);
#endif
}
    
void
print_message(char *mesg)
{
  if (message_widget_up) {
    popup_message(mesg);
    if (debug)
      fprintf(stderr, "#%s\n", mesg);
  } else {
    fprintf(stderr, "%s\n", mesg);
    fflush(stderr);
  }
}






