/*
 * Sentry Video System
 *
 * $Id: $
 */

/*
 * udprecv.c - recieve udp packets and reassemble them.
 */

#include <stdio.h>
#include <netinet/in.h>
#include <errno.h>

#include "sentry.h"

extern int debug;
extern int errno;

/*
 * data packet offset list
 */
struct image_offset_t {
  struct image_offset_t *next;
  int offset;
  int length;
};

/*
 * linear buffer for udp packet
 */
#define LBUF_SIZE       10
#define OLIST_SIZE      31
typedef struct {
  int valid;            /* valid flag (1:used, 0:free) */
  u_long ts;            /* timestamp */
  struct image_offset_t olist[OLIST_SIZE+1]; /* offset list */
  caddr_t pimage;       /* pointer for image */
  int size;             /* image size */
  int sum;              /* summing of store */
} lbuf_t;

static lbuf_t lbuf[LBUF_SIZE];

void
refresh_lbuf(int lbuf_id)
{

  /* lbuf valid flag off */
  lbuf[lbuf_id].valid = 0;

  /* timestamp clear */
  lbuf[lbuf_id].ts = 0;

  /* summing initialize */
  lbuf[lbuf_id].sum = 0L;

  memset(lbuf[lbuf_id].olist, 0,
	 sizeof(struct image_offset_t)*(OLIST_SIZE+1));

  if (debug >= 100)
    fprintf(stderr, "refresh_lbuf: lbuf[%d] refreshed.\n", lbuf_id);

  return;
}

void
clear_lbuf(int lbuf_id)
{
  refresh_lbuf(lbuf_id);

  lbuf[lbuf_id].pimage = NULL;
  lbuf[lbuf_id].size = 0;

  return;
}

int 
find_lbuf(sentry_image_header *hdr)
{
  int lbuf_id;
  u_long ts = hdr->hd_ts;

  for(lbuf_id=0; lbuf_id<LBUF_SIZE; lbuf_id++) {
    if (lbuf[lbuf_id].valid == 1 && lbuf[lbuf_id].ts == ts) {
      if (debug >= 1000)
	fprintf(stderr, "find_lbuf: find_lbuf[%d].ts=%u/ts=%u\n",
		lbuf_id, lbuf[lbuf_id].ts, ts);
      return lbuf_id;
    }
  }

  if (debug >= 100)
    fprintf(stderr, "find_lbuf: cannot find lbuf_id/ ts=%u\n", ts);

  return -1;
}

int
get_lbuf(void)
{
  int lbuf_id, oldest_id = -1;
  u_long oldest_ts;

  for(lbuf_id=0; lbuf_id<LBUF_SIZE; lbuf_id++) {
    if (debug >= 1000)
      fprintf(stderr, "get_lbuf: lbuf[%d].valid=%d [old[%d].ts=%u/ lbuf[%d].ts=%u\n",
	      lbuf_id, lbuf[lbuf_id].valid, oldest_id, oldest_ts,
	      lbuf_id, lbuf[lbuf_id].ts);

    if (lbuf[lbuf_id].valid == 0) {
      lbuf[lbuf_id].valid = 1;
      return lbuf_id;
    } else {
      if (oldest_id == -1 || oldest_ts > lbuf[lbuf_id].ts) {
	oldest_id = lbuf_id;
	oldest_ts = lbuf[lbuf_id].ts;
      }
    }
  }

  refresh_lbuf(oldest_id);
  lbuf[oldest_id].valid = 1;
  return oldest_id;
}

void
init_lbuf(void)
{
  int lbuf_id;

  for(lbuf_id=0; lbuf_id<LBUF_SIZE; lbuf_id++) {
    clear_lbuf(lbuf_id);
  }

  if (debug >= 100)
    fprintf(stderr, "init_lbuf: all lbuf cleared.\n");
}

int
show_lbuf_offs(int lbuf_id)
{
  int off_id;

  fprintf(stderr, "id# %3d: %7s", lbuf_id, "");
  for(off_id=0; off_id<OLIST_SIZE;off_id++) {
    fprintf(stderr, "%7d:%7d ",
	    lbuf[lbuf_id].olist[off_id].offset, 
	    lbuf[lbuf_id].olist[off_id].length);
  }
  fprintf(stderr, "\n");
}

int
join_lbuf(int lbuf_id, int part_off, int part_len)
{
  int off_id,pos;
  int prev, next;
  int ret=-1;

#if 0
  fprintf(stderr, "join_lbuf: id %3d, off %7d, len %7d -> %7d\n",
	  lbuf_id, part_off, part_len, part_off+part_len);
#endif
#if 0
  show_lbuf_offs[lbuf_id);
#endif

  pos = -1;
  for(off_id=0; off_id<OLIST_SIZE;off_id++) {
    if (off_id > 0  && lbuf[lbuf_id].olist[off_id].offset == 0 &&
	lbuf[lbuf_id].olist[off_id].length == 0) {
      pos = off_id;
      break;
    }
    if (lbuf[lbuf_id].olist[off_id].offset < part_off) {
#if 0
      fprintf(stderr, "\tskip\n");
#endif
    } else {
      pos = off_id;
      break;
    }
  }
#if 0
  fprintf(stderr, "\tpos %d\n", pos);
#endif

  prev = pos - 1;
  next = pos + 1;

  if (prev >= 0 && 
      lbuf[lbuf_id].olist[prev].offset +
      lbuf[lbuf_id].olist[prev].length == part_off) {
    lbuf[lbuf_id].olist[prev].length += part_len;
    ret = 0;
#if 0
    fprintf(stderr, "\treplace\n");
#endif
#if 0
    if (debug >= 5)
	fprintf(stderr, ".");
#endif
    goto out;
  }

  memmove(&lbuf[lbuf_id].olist[pos+1], &lbuf[lbuf_id].olist[pos],
	  sizeof(struct image_offset_t)*(OLIST_SIZE-pos));
  
  lbuf[lbuf_id].olist[pos].offset = part_off;
  lbuf[lbuf_id].olist[pos].length = part_len;
#if 0
  fprintf(stderr, "\tnew\n");
#endif
#if 0
  if (debug >= 5)
    fprintf(stderr, "*");
#endif

  ret = 1;

out:
#if 0
  show_lbuf_offs[off_id];
#endif
#if 0
  fprintf(stderr, "\t#%d off %c len %d\n",
	  lbuf_id, lbuf[lbuf_id].olist[pos].offset, 
	  lbuf[lbuf_id].olist[pos].length);
  fprintf(stderr, "\tret %d\n\n", ret);
#endif

  return ret;
}


/*
 * udp_reassemble - receive udp packets, and order and reassemble them.
 *     NOTE: udp packets are arrived out of order.
 */
int
udp_reassemble(sentry_image_header *hdr, char *buf, int buflen, char **image)
{
  int lbuf_id;

  if ((lbuf_id = find_lbuf(hdr)) < 0) {
    lbuf_id = get_lbuf();

    lbuf[lbuf_id].ts = hdr->hd_ts;
  }

  /*
   * allocate new memory
   */
  if (lbuf[lbuf_id].size == 0 || lbuf[lbuf_id].size < hdr->size) {
    if (debug >= 1000) 
      fprintf(stderr, "udp_reassmble: size %d (%d x %d) format %d:\n",
	      hdr->size, hdr->width, hdr->height, hdr->format);

    printf("new buffer %2d (new %ld, old %ld),\n",
	   lbuf_id, (long)hdr->size, (long)lbuf[lbuf_id].size);

    if (lbuf[lbuf_id].pimage != NULL) {
      printf("resize buffer %2d.\n", lbuf_id);

      free(lbuf[lbuf_id].pimage);
      lbuf[lbuf_id].size = 0;
    }
    lbuf[lbuf_id].pimage = (caddr_t)malloc(hdr->size+K(1));
    lbuf[lbuf_id].size = hdr->size+K(1);
  }
  *image = lbuf[lbuf_id].pimage;
  if (debug >= 100)
    fprintf(stderr, "udp_reassemble: *image=%d\n", *image);

  /*
   * packet reassemble
   */
  if ((hdr->hd_off + buflen) > hdr->size) {
    buflen = hdr->size - hdr->hd_off;
    fprintf(stderr, "udp_reassemble: offset something wrong.\n");
  }

  /*
   * store part of image
   */
  memcpy((char *)&(*image)[hdr->hd_off], buf, buflen);
  if (debug >= 1000)
    fprintf(stderr, "udp_reassemble: copy packets to buffer.\n");
  lbuf[lbuf_id].sum += buflen;


  join_lbuf(lbuf_id, hdr->hd_off, buflen);
#if 0
  printf("\t#0 offset %d, length %d\n",
	 lbuf[lbuf_id].olist[0].offset, lbuf[lbuf_id].olist[0].length);
#endif
  if (lbuf[lbuf_id].olist[0].offset == 0 &&
      lbuf[lbuf_id].olist[0].length == hdr->size) {
    /*
     * udp reassemble completed (all packets are recieved)
     */
#if 0
    if (debug >= 5) {
      fprintf(stderr, "\n");
    } else if (debug) {
      fprintf(stderr, "\t*** success *** %8d\n", lbuf[lbuf_id].sum);
    }
#endif
#if 0
    fprintf(stderr, "\tsum %d\n", lbuf[lbuf_id].sum);
#endif

    return lbuf[lbuf_id].sum;
  } 
  else {
    /*
     * more packets to receive
     */
#if 0
    fprintf(stderr, "\tmiss\n");
#endif
    return 0;
  }
}
  



      
	 
