/*
 * Sentry Video System
 *
 * $Id: $
 */

/*
 * sentry.h - sentry header file
 */
#ifndef _SENTRY_H_
#define _SENTRY_H_

#include <sys/types.h>

extern int debug;
extern int errno;

#define SENTRY_DIRLEN   (256)

#ifndef K
#define K(x)            ((x)*1024L)
#endif
#ifndef M
#define M(x)            ((x)*1024L*1024L)
#endif
#ifndef TOK
#define TOK(x)          ((x)/1024L)
#endif
#ifndef TOM
#define TOM(x)          ((x)/(1024L*1024L))
#endif


/*
 * address and port
 */
#define SENTRY_MULTICAST_ADDR   "224.0.0.1"
#define SENTRY_CTRL_PORT        7001
#define SENTRY_DATA_PORT        7000

/*
 * sentry version
 */
#define SENTRY_VERSION          2

/*
 * sentry request
 */
#define SENTRY_UNICAST_OPEN     (0xF0)
#define SENTRY_MULTICAST_OPEN   (0xF1)
#define SENTRY_CLOSE            (0xF2)

#define QOS_REQUEST_SIZE        (0x01)
#define QOS_REQUEST_ZOOM        (0x02)
#define QOS_REQUEST_COLOR       (0x04)
#define QOS_REQUEST_RATE        (0x08)
#define QOS_REQUEST_ENCODE      (0x10)

/*
 * normal vision ratio (4:3)
 */
#define SIZE_640x480            (1)
#define SIZE_320x240            (2)
#define SIZE_212x160            (3)
#define SIZE_160x120            (4)
/*
 * wide vision ratio (16:9)
 */
#define SIZE_640x360            (5)
#define SIZE_320x180            (6)
#define SIZE_212x120            (7)
#define SIZE_160x90             (8)

#define FULL_COLOR              (1)
#define MONOCHROME              (2)

#define ENCODE_NONE             (0)
#define ENCODE_JPEG             (1)

typedef struct {
#if 0
  int version;                  /* sentry version */
#endif
  int type;                     /* request type */
  
  int size;                     /* frame size */
  int zoom;                     /* zoom ratio */
  int color;                    /* color type */
  int encode;                   /* encode type */
  int rate;                     /* frame rate */
} sentry_request;


/*
 * sentry image header
 */
typedef struct {
  int hd_ver;                  /* sentry version */
  u_long hd_ts;                /* timestamp */
  int hd_off;                  /* image offset */

  int size;                    /* image size (xsize * ysize * color_space) */
  int width;                   /* image xsize */
  int height;                  /* image ysize */
  int format;                  /* color type */
  int encode;                  /* encode type */
} sentry_image_header;


#define MAX_DGRAM_LEN          K(16)
#define SENTRY_HEADER_LEN      sizeof(sentry_image_header)
#define SENTRY_MAX_DATALEN     (MAX_DGRAM_LEN - SENTRY_HEADER_LEN)


#endif /* _SENTRY_H_ */
  
