/*
 * Sentry video system
 *   is controlled qos according to client's requests.
 *
 *        original by akira-y@is.aist-nara.ac.jp, 1994, 1995
 *        modified by k-chinen@is.aist-nara.ac.jp, 1996
 *        modified by satosi-f@is.aist-nara.ac.jp, 1997
 *
 *  $Id: get_request.c,v 1.1 1997/08/01 20:29:03 satosi-f Exp $
 */

/*
 * get_request.c - get request
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <signal.h>
#include <errno.h>

#include "sentry.h"
#include "sentry_param.h"

extern int errno;
extern int debug;
extern int verbose;

/* global */
struct {
  int type;                /* qos type (QOS_ ZOOM, SIZE, COLOR, RATE,...) */
  int val;                 /* qos value */
  int change_flag;         /* qos change flag */
} req_val[QOS_PARAMS];


void
init_req_val(void)
{
  int type;

  for(type=0; type<QOS_PARAMS; type++) {
    /*
    req_val[type].type = type;
    */
    req_val[type].val = 0;
    req_val[type].change_flag = 0;

    if (debug > 10)
      fprintf(stderr, "init_req_val: type=%d initialized done.\n", type);
  }
}

#if 0
int
get_qos_id(int type)
{
  int qos_id;

  for(qos_id=0; qos_id<QOS_PARAMS; qos_id++) {
    if (req_val[qos_id].type == type)
      return qos_id;
  }
  return -1;
}
#endif

int
get_qos_val(int type)
{
  if (debug > 10)
    fprintf(stderr, "get_qos_val: req_val[%d]=%d\n",
            type, req_val[type].val);

  if (type >= 0 && type < QOS_PARAMS)
    return req_val[type].val;
  return -1;
}


int
set_qos(int type, int val)
{
#if 0
  extern void set_rate_interval();

  if (debug)
    fprintf(stderr, "type=%d\n", type);

  if (type == QOS_RATE) {
    if (1)
      fprintf(stderr, "set_qos: req_val[%d].val=%d/ val=%d\n",
	      type, req_val[type].val, val);

    if (req_val[type].val < val) {
      req_val[type].val = val;
      set_rate_interval();

      if (1)
	fprintf(stderr, "set_qos: req_val[%d].val=%d\n",
		type, req_val[type].val);
    }

    return type;
  }
#endif
    
  if (type >= 0 && type < QOS_PARAMS) {
    if (req_val[type].val != val) {
      req_val[type].val = val;
      req_val[type].change_flag = 1;

    if (debug)
      fprintf(stderr, "set_qos: req_val[%d].val=%d\n",
              type, req_val[type].val);
    }
  }
  return type;
}

int
check_qos_change(int type)
{
  if (type >= 0 && type < QOS_PARAMS)
    return req_val[type].change_flag;
  return 0;
}

void
clear_qos_change(int type)
{
  if (type >= 0 && type < QOS_PARAMS)
    req_val[type].change_flag = 0;
}


int
build_dataport(struct sockaddr_in *cli)
{
  int datafd;
  extern int build_udp_port();

  if ((datafd = build_udp_port(0)) < 0)
    return -1;

  if (connect(datafd, (struct sockaddr *)cli,
              sizeof(struct sockaddr_in)) < 0) {
    close(datafd);
    return -1;
  }

  return datafd;
}

#ifdef LOGTIME
extern time_t logtime;
extern struct tm *log_tm;
#endif

int
recv_tcp_request(int sockfd)
{
  int nread, readlen;
  char *readptr;
  sentry_request get_req_val;             /* sentry request packet */
  int datafd;                             /* data socket fd */
  int cid, linkid;                        /* client id */
  struct sockaddr_in client_addr, *cli;   /* clinet address */
  u_char ttl;                             /* multicast ttl */

  extern int num_client, maxwritefds;
  extern fd_set readmask, writemask;      /* socket read/write mask */
  extern int find_client_by_fd(), find_client_by_addr(), add_client();
  extern u_int get_client_flag();
  extern struct sockaddr_in * get_client_sockaddr();
  extern int get_client_sockfd();
  /* extern void clear_client(); */
  extern void sentry_close();
  extern void rate_set();
#ifdef SUPPORT_MULTIRATE
  extern void set_client_rate();
#endif

  readptr = (char *)&get_req_val;
  for(readlen = 0; readlen < sizeof(get_req_val); readlen += nread) {

    nread = read(sockfd, readptr, sizeof(get_req_val) - readlen);
    if (debug > 10)
      fprintf(stderr, "get_request: nread=%d\n", nread);

    if (nread < 0) {
      if (errno == EINTR)
        continue;
      else {
        fprintf(stderr, "recv_tcp_request: read error.\n");
        return -1;
      }
    } else if (nread == 0) {
      (void)sentry_close(sockfd);
      return 0;
    }
    readptr += nread;
  }
  /*
   * translate netbyte-order to hostbyte-order
   */
  get_req_val.type = get_req_val.type;
  get_req_val.zoom = get_req_val.zoom;
  get_req_val.size = get_req_val.size;
  get_req_val.color = get_req_val.color;
  get_req_val.rate = get_req_val.rate;
  get_req_val.encode = get_req_val.encode;

  if (verbose) {
#ifdef LOGTIME
    logtime = time(NULL);
    log_tm = localtime(&logtime);
    fprintf(stderr, "[%02d/%02d/%02d %02d:%02d:%02d] ",
	    log_tm->tm_year, log_tm->tm_mon+1, log_tm->tm_mday,
	    log_tm->tm_hour, log_tm->tm_min, log_tm->tm_sec);
#endif
    fprintf(stderr, "sentryd: request [val=%d] is arrived from [socket=%d].\n",
            get_req_val.type, sockfd);
  }

  /*
  cid = find_client_by_fd(sockfd);
  if (debug > 100)
    fprintf(stderr, "recv_tcp_request: find client (cid=%d)\n", cid);
    */
  switch (get_req_val.type) {
  case SENTRY_UNICAST_OPEN:
    cid = find_client_by_fd(sockfd);
    if (debug > 100)
      fprintf(stderr, "get_request: client[%d].flag=%d\n",
              cid, get_client_flag(cid));

    if (get_client_flag(cid) & DATA_SOCKET) {
      /*
       * both ctrl port and data port are same TCP port
       */
      FD_SET(sockfd, &writemask);
      maxwritefds = MAX(maxwritefds, sockfd);
      /* no-op */
    } else if (get_client_flag(cid) & CTRL_SOCKET) {
      /*
       * ctrl port is TCP port, and data port is UDP port
       */
      cli = (struct sockaddr_in *)get_client_sockaddr(cid);
      if (cli == NULL) {
        fprintf(stderr, "recv_tcp_request: cannot get client address.\n");
        break;
      }
      bcopy((char *)cli, &client_addr, sizeof(client_addr));
      client_addr.sin_port = htons(SENTRY_DATA_PORT);
      if (find_client_by_addr(&client_addr) >= 0)
        break;
      if ((datafd = build_dataport(&client_addr)) < 0 ||
          add_client(DATA_SOCKET, datafd, &client_addr) < 0) {
        close(datafd);
        break;
      }
      FD_SET(datafd, &writemask);
      maxwritefds = MAX(maxwritefds, datafd);
      num_client++;

      set_client_link_datafd(find_client_by_fd(sockfd), datafd);
      set_client_link_nctrlfds(find_client_by_fd(datafd));

      if (1) {
        fprintf(stderr, "get_request: SENTRY_UNICAST_OPEN: [zoom=%d/ size=%d/ color=%d/ rate=%d]\n",
                get_req_val.zoom, get_req_val.size,
                get_req_val.color, get_req_val.rate);
      }

      set_qos(QOS_ZOOM, get_req_val.zoom);
      set_qos(QOS_SIZE, get_req_val.size);
      set_qos(QOS_COLOR, get_req_val.color);
      if (get_qos_val(QOS_RATE) < get_req_val.rate) {
	set_qos(QOS_RATE, get_req_val.rate);
	set_rate_interval();
	clear_qos_change(QOS_RATE);
      }
#ifdef SUPPORT_MULTIRATE
      set_client_rate(find_client_by_fd(datafd), get_req_val.rate);
#endif

      if (!check_video(NULL))
        (void)init_video(NULL);
      (void)rate_set();

      printf("UNICAST_OPEN ready.\n");
    } else {
      /* no-op */
    }
    break;

  case SENTRY_MULTICAST_OPEN:
    cid = find_client_by_fd(sockfd);
    if (get_client_flag(cid) & DATA_SOCKET) {
      /*
       * both ctrl port and data port are same TCP port.
       * in this case, multicast_open request is ignored.
       */
      return 1;
    }
    client_addr.sin_family = AF_INET;
    client_addr.sin_addr.s_addr = htonl(inet_addr(SENTRY_MULTICAST_ADDR));
    client_addr.sin_port = htons(SENTRY_DATA_PORT);
    cid = find_client_by_addr(&client_addr);
    if (get_client_flag(cid) & DATA_SOCKET) {
      /*
       * multicast data socket is already existing.
       */
    } else {
      /*
       * first request for creating multicast data socket.
       */
      if ((datafd = build_dataport(&client_addr)) < 0 ||
          (cid = add_client(DATA_SOCKET, datafd, &client_addr)) < 0) {
        close(datafd);
        break;
      }
#if 0
      ttl = 16;
      if (setsockopt(datafd, IPPROTO_IP, IP_MULTICAST_TTL,
                     (char *)&ttl, sizeof(ttl)) < 0) {
        fprintf(stderr, "get_request: cannot set IP_MULTICAST_TTL: ttl=%u\n",
                ttl);
        perror("get_request: IP_MULTICAST_TTL");
        close(datafd);
        break;
      }
#endif

      FD_SET(datafd, &writemask);
      maxwritefds = MAX(maxwritefds, datafd);
      num_client++;

      (void)set_client_link_datafd(find_client_by_fd(sockfd), datafd);
      (void)set_client_link_nctrlfds(find_client_by_fd(datafd));

      set_qos(QOS_ZOOM, get_req_val.zoom);
      set_qos(QOS_SIZE, get_req_val.size);
      set_qos(QOS_COLOR, get_req_val.color);
      if (get_qos_val(QOS_RATE) < get_req_val.rate) {
	set_qos(QOS_RATE, get_req_val.rate);
	set_rate_interval();
	clear_qos_change(QOS_RATE);
      }
#ifdef SUPPORT_MULTIRATE
      set_client_rate(find_client_by_fd(datafd), get_req_val.rate);
#endif

      if (!check_video(NULL))
        (void)init_video(NULL);
      (void)rate_set();

      printf("MULTICAST_OPEN ready.\n");
    }

    /*
     * unnecessary to announce multicast address,
     * because the port number is well-known.
     (void)send_response(SENTRY_ADDR_ANNOUNCE, sockfd, datafd);
     ***/

    break;

  case SENTRY_CLOSE:
    (void)sentry_close(sockfd);

#if 0
    cid = find_client_by_fd(sockfd);
    datafd = get_client_link_datafd(cid);
    linkid = find_client_by_fd(datafd);
    (void)clear_client(linkid);
    if (get_client_flag(linkid) == INVALID) {
      close(datafd);
      FD_CLR(datafd, &writemask);
    }

    (void)clear_client(cid);
    if (get_client_flag(cid) == INVALID) {
      close(sockfd);
      FD_CLR(sockfd, &readmask);
    }
#endif

    break;

  case QOS_REQUEST_ZOOM:
    set_qos(QOS_SIZE, get_req_val.zoom);
    break;

  case QOS_REQUEST_SIZE:
    set_qos(QOS_SIZE, get_req_val.size);
    break;

  case QOS_REQUEST_COLOR:
    set_qos(QOS_COLOR, get_req_val.color);
    break;

  case QOS_REQUEST_RATE:
    if (get_qos_val(QOS_RATE) < get_req_val.rate) {
      set_qos(QOS_RATE, get_req_val.rate);
      set_rate_interval();
      clear_qos_change(QOS_RATE);
    }
#ifdef SUPPORT_MULTIRATE
    set_client_rate(find_client_by_fd(sockfd), get_req_val.rate);
#endif

    break;

  default:
    fprintf(stderr, "sentryd: unknown request type. ignored.\n");
  }
  return 0;
}

void
sentry_close(int sockfd)
{
  int cid, linkid, datafd;
  struct sockaddr_in client_addr;
  extern fd_set readmask, writemask;
  extern int num_client, maxwritefds;
  extern int find_client_by_fd(), get_client_link_datafd();
  extern u_int get_client_flag();
  extern void clear_client();

  cid = find_client_by_fd(sockfd);
  datafd = get_client_link_datafd(cid);
  linkid = find_client_by_fd(datafd);

  (void)clear_client(linkid);
  if (get_client_flag(linkid) == INVALID) {
    shutdown(datafd, 2);
    close(datafd);
    FD_CLR(datafd, &writemask);
    if (--num_client <= 0) {
      maxwritefds = 0;
      signal(SIGALRM, SIG_IGN);
    }
    if (debug)
      fprintf(stderr, "sentry_close: [datafd=%d] closed.\n", datafd);
  }
  (void)clear_client(cid);
  if (get_client_flag(cid) == INVALID) {
    shutdown(sockfd, 2);
    close(sockfd);
    FD_CLR(sockfd, &readmask);
    if (verbose) {
#ifdef LOGTIME
      logtime = time(NULL);
      log_tm = localtime(&logtime);
      fprintf(stderr, "[%02d/%02d/%02d %02d:%02d:%02d] ",
	      log_tm->tm_year, log_tm->tm_mon+1, log_tm->tm_mday,
	      log_tm->tm_hour, log_tm->tm_min, log_tm->tm_mday);
      fprintf(stderr, "connection [sockfd=%d] closed.\n", sockfd);
#endif
      fprintf(stderr, "sentry_close: connection [sockfd=%d] closed.\n",
              sockfd);
    }
  }

}

static struct itimerval sendtimer;
static struct timeval start_tv = {0, 0};

void
rate_set(void)
{
  int sec, usec;
  int interval;
  int rate;
  static int FrameCount = 0;
  static int TotalCount = -1;
  struct timeval tv;
  extern void send_image();
  extern int get_qos_val();
  extern int tvsub();

  interval = 100000000 / get_qos_val(QOS_RATE);
  sec = interval / 1000000;
  usec = (interval % 1000000) * 1;

  if (debug > 10)
    fprintf(stderr, "rate_set: interval=%d.%06d (sec)\n", sec, usec);

  sendtimer.it_interval.tv_sec = 0;
  sendtimer.it_interval.tv_usec = 0;
  sendtimer.it_value.tv_sec = sec;
  sendtimer.it_value.tv_usec = usec;

  if (start_tv.tv_sec == 0)
    gettimeofday(&start_tv, NULL);
  gettimeofday(&tv, NULL);
  tvsub(&tv, &start_tv);

  if (++TotalCount >= 10000) {
    printf("rate_set: Total [frame=%d/%d.%06d]\n",
            FrameCount, tv.tv_sec, tv.tv_usec);
    TotalCount = 0;
  }
  FrameCount ++;

  if (debug > 10)
    fprintf(stderr, "rate_set: interval=%d.%06d\n",
            sendtimer.it_value.tv_sec, sendtimer.it_value.tv_usec);

  signal(SIGALRM, send_image);
  setitimer(ITIMER_REAL, &sendtimer, NULL);
}

int
tvsub(struct timeval *tv1, struct timeval *tv2)
{
  if ((tv1->tv_usec -= tv2->tv_usec) < 0) {
    tv1->tv_usec += 1000000;
    tv1->tv_sec -= 1;
  }
  tv1->tv_sec -= tv2->tv_sec;
}




