/*
 * Sentry video system
 *   is controlled qos according to client's requests.
 *
 *        original by akira-y@is.aist-nara.ac.jp, 1994, 1995
 *        modified by k-chinen@is.aist-nara.ac.jp, 1996
 *        modified by satosi-f@is.aist-nara.ac.jp, 1997
 *
 *  $Id: image_encode.c,v 1.1 1997/08/01 20:30:18 satosi-f Exp satosi-f $
 */

/*
 * image_encode.c - image encode procedures (jpeg, ...)
 */

#include <stdio.h>
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/time.h>

#include <dmedia/cl.h>

#include "sentry.h"
#include "sentry_param.h"

extern int errno;
extern int debug;

int
encode(char *cur, int size, int xsize, int ysize, int *nxsize, int *nysize, char **new)
{
  register u_long *s, *d;
  register int i,j;
  char *dst;
  int newsize = size / 4;
  int pixelsize = size / 4;

  if (debug)
    fprintf(stderr, "encode: size=%d\n", size);

#if 0
  s = (u_long *)cur;
  dst = (char *)malloc(newsize);
  d = (u_long *)dst;

  i = j = 0;
  while (1) {
    d[i] = s[j];
    j += 2;
    if (j >= pixelsize)
      break;
    i ++;
  }

  if (debug)
    fprintf(stderr, "encode: i=%d/ pixelsiez=%d\n", i, pixelsize);

  *nxsize = xsize / 2;
  *nysize = ysize / 2;
  *new = dst;
  return newsize;
#else
  *new = cur;
  *nxsize = xsize;
  *nysize = ysize;
  return size;
#endif
}

static int jpeg_system_initted = 0;

CLhandle compressorHdl;

void *inner_buff=NULL;

int
init_jpeg_system(int x, int y)
{
  int chk;

  jpeg_system_initted = 1;

  chk = clOpenCompressor(CL_JPEG_COSMO, &compressorHdl);
  if (chk != SUCCESS) {
    fprintf(stderr, "init_jpeg_system: no COSMO system.\n");

    chk = clOpenCompressor(CL_JPEG_SOFTWARE, &compressorHdl);
    if (chk != SUCCESS) {
      fprintf(stderr, "init_jpeg_system: no JPEG library.\n");
      return -1;
    } else
      fprintf(stderr, "init_jpeg_system: use jpeg software encoding.\n");
  } else {
    fprintf(stderr, "init_jpeg_system: have COSMO system.\n");
  }

  clSetParam(compressorHdl, CL_IMAGE_WIDTH, x);
  clSetParam(compressorHdl, CL_IMAGE_HEIGHT, y);
  clSetParam(compressorHdl, CL_FORMAT, CL_FORMAT_XBGR);

  if (inner_buff != NULL) {
    free(inner_buff);
    inner_buff = NULL;
  }

  if ((inner_buff = malloc(x*y*5)) == NULL) {
    fprintf(stderr, "init_jpeg_system: no memory (%d)\n", errno);
    return -1;
  }

  return 0;
}


int
encode_jpeg(char *cur, int size, int xsize, int ysize, int *nxsize, int *nysize, char **new)
{
  int csize;
#if 0
  struct timeval tv1;

  gettimeofday(&tv1, NULL);
  fprintf(stderr, "encode_jpeg: %d.%06d -> ", tv1.tv_sec, tv1.tv_usec);
#endif

  if (jpeg_system_initted == 0) {
    init_jpeg_system(xsize, ysize);
  }

  csize = xsize * ysize * 5;

  if (clCompress(compressorHdl, 1, (void *)cur, &csize, inner_buff) < 0) {
    fprintf(stderr, "encode_jpeg: clCompress() error.\n");
  }
#if 0
  gettimeofday(&tv1, NULL);
  fprintf(stderr, "%d.%06d\n", tv1.tv_sec, tv1.tv_usec);
#endif

  if (debug > 5)
    fprintf(stderr, "encode_jpeg: size %d -> %d\n", size, csize);

  *new = inner_buff;

  *nxsize = xsize;
  *nysize = ysize;

  return csize;
}

