/*
 * Sentry video system
 *   is controlled qos according to client's requests.
 *
 *        original by akira-y@is.aist-nara.ac.jp, 1994, 1995
 *        modified by k-chinen@is.aist-nara.ac.jp, 1996
 *        modified by satosi-f@is.aist-nara.ac.jp, 1997
 *
 *  $Id: main.c,v 1.1 1997/08/01 20:30:18 satosi-f Exp satosi-f $
 */

/*
 * main.c - main
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/signal.h>
#include <sys/wait.h>
#include <time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <syslog.h>
#include <errno.h>

#include "sentry.h"
#include "sentry_param.h"

int debug;
int verbose = 1;

#define QLEN 5

/* global */
int num_client;
int maxreadfds, maxwritefds;       /* max read/write fds */
fd_set readmask, writemask;        /* socket mask */

#ifdef LOGTIME
time_t logtime;
struct tm *log_tm;
#endif

extern int num_client;


int
bind_socket(int sockfd, int port)
{
  struct sockaddr_in serv_addr;

  bzero((char *)&serv_addr, sizeof(serv_addr));
  serv_addr.sin_family = AF_INET;
  serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  serv_addr.sin_port = htons(port);

  if (bind(sockfd, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) < 0) {
    fprintf(stderr, "sentryd: cannot bind port %d.\n", port);
    close(sockfd);
    return -1;
  }
  return 0;
}

int
build_tcp_port(int port, int qlen)
{
  int sockfd;
  int flag=1;

  if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    fprintf(stderr, "sentryd: cannot make new TCP socket.\n");
    return -1;
    exit (1);
  }

  if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR,
                 (char *)&flag, sizeof(flag)) < 0) {
    fprintf(stderr, "sentryd: cannot set REUSEADDR\n");
    return -1;
  }

  if (bind_socket(sockfd, port) < 0) {
    fprintf(stderr, "sentryd: please wait several seconds.\n");
    close(sockfd);
    return -1;
    exit (1);
  }
  listen(sockfd, 5);
  if (debug)
    fprintf(stderr, "sentryd: new tcp socket %d [port=%d]\n", sockfd, port);

  return sockfd;
}

int build_udp_port(int port)
{
  int sockfd;
  int flag=1;
  int flaglen = sizeof(flag);

  if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    fprintf(stderr, "sentryd: cannot make new UDP socket.\n");
    return -1;
    exit (1);
  }

  if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR,
                 (char *)&flag, sizeof(flag)) < 0) {
    fprintf(stderr, "sentryd: cannot set REUSEADDR\n");
    return -1;
  }

#if 1
  /*
  flag = 16384000;
  */
  flag = K(16000);
  if (setsockopt(sockfd, SOL_SOCKET, SO_SNDBUF,
                 (char *)&flag, flaglen) < 0) {
    fprintf(stderr, "sentryd: cannot set SNDBUF size.\n");
    return -1;
  }
#endif

  if (1) {
    if (getsockopt(sockfd, SOL_SOCKET, SO_SNDBUF,
                   (char *)&flag, &flaglen) < 0) {
      fprintf(stderr, "sentryd: cannot get SNDBUF size.\n");
      return -1;
    } else {
      fprintf(stderr, "sentryd: SNDBUF size = %d\n", flag);
    }
  }

  if (bind_socket(sockfd, port) < 0) {
    fprintf(stderr, "sentryd: please wait several seconds.\n");
    close(sockfd);
    return -1;
    exit (1);
  }
  if (debug)
    fprintf(stderr, "sentryd: new udp socket %d [port=%d]\n", sockfd, port);

  return sockfd;
}

void
reaper()
{
  union wait status;
  while(wait3((int *)&status, WNOHANG, (struct rusage *)0) >= 0)
    ;
}

void
finish_sentryd() {
  char logstring[256];

  if (verbose) {
#ifdef LOGTIME
    logtime = time(NULL);
    log_tm = localtime(&logtime);
    fprintf(stderr, "[%02d/%02d/%02d %02d:%02d:%02d] ",
	    log_tm->tm_year, log_tm->tm_mon+1, log_tm->tm_mday,
	    log_tm->tm_hour, log_tm->tm_min, log_tm->tm_sec);
#endif
    fprintf(stderr, "finish sentryd\nBye.\n");
  }

  sprintf(logstring, "sentryd is stopped.\n");
  syslog(LOG_NOTICE, logstring);
  closelog();

  exit (0);
}

void
mainloop(int ctrlfd, int datafd)
{
  int fd;
  extern int maxreadfds;
  fd_set readfds;
  extern fd_set readmask;
  struct sockaddr_in client_addr;
  int client_addr_len = sizeof(client_addr);
  extern void init_client();

  (void)init_req_val();
  (void)init_client();

  FD_ZERO(&readmask);
  FD_SET(ctrlfd, &readmask);
  FD_SET(datafd, &readmask);
  maxreadfds = MAX(ctrlfd, datafd);

  while (1) {
    readfds = readmask;

    if (select(maxreadfds+1, &readfds, (fd_set *)0, (fd_set *)0,
               (struct timeval *)0) < 0) {
      if (errno == EINTR)
        continue;
      fprintf(stderr, "sentryd: mainloop select error(%d).\n", errno);
      perror("select");
      if (errno == EBUSY)
        continue;
      exit (1);
    }

    /*
     * TCP accept ctrl socket
     * (ctrl port is TCP port, data port is TCP port)
     */
    if (FD_ISSET(ctrlfd, &readfds)) {
      int newsockfd;  /* TCP socket (for ctrl) */

      newsockfd = accept(ctrlfd, (struct sockaddr *)&client_addr,
                         &client_addr_len);
      if (newsockfd < 0) {
        if (errno == EINTR)
          continue;
        fprintf(stderr, "sentryd: mainloop ctrl socket accept error.\n");
        exit (1);
      }

      if (add_client(CTRL_SOCKET, newsockfd, &client_addr) < 0) {
        fprintf(stderr, "close connection %d.\n", newsockfd);
        shutdown(newsockfd, 2);
        close(newsockfd);
      } else {
        FD_SET(newsockfd, &readmask);
        maxreadfds = MAX(maxreadfds, newsockfd);

        fprintf(stdout, "sentryd: [%s/%d] is connected [socket=%d] to ctrlfd.\n",
                inet_ntoa(client_addr.sin_addr), ntohs(client_addr.sin_port),
                newsockfd);
      }
    }

    /*
     * TCP accept data socket
     * (both ctrl port and data port are TCP port)
     */
    if (FD_ISSET(datafd, &readfds)) {
      int newsockfd;  /* TCP slave socket (for data) */

      newsockfd = accept(datafd, (struct sockaddr *)&client_addr,
                         &client_addr_len);
      if (newsockfd < 0) {
        if (errno == EINTR)
          continue;
        fprintf(stderr, "sentryd: mainloop data socket accept error.\n");
        exit (1);
      }

      if (add_client((CTRL_SOCKET|DATA_SOCKET), newsockfd,
                     &client_addr) < 0) {
        fprintf(stderr, "close connection %d.\n", newsockfd);
        shutdown(newsockfd, 2);
        close(newsockfd);
      } else {
        FD_SET(newsockfd, &readmask);
        maxreadfds = MAX(maxreadfds, newsockfd);

        fprintf(stdout, "sentryd: [%s/%d] is connected [socket=%d] to datafd.\n",
                inet_ntoa(client_addr.sin_addr), ntohs(client_addr.sin_port),
                newsockfd);
      }
    }

    /* requests are arrived */
    for (fd=0; fd<=maxreadfds; fd++) {
      if (fd != ctrlfd && fd != datafd && FD_ISSET(fd, &readfds))

        recv_tcp_request(fd);
    }
  } /* end-of-while */
  /* NOT REACHED */
}

extern int sentry_max_datalen;
extern int require_encode_type;

int
main(int argc, char **argv)
{
  int ctrl_port = SENTRY_CTRL_PORT;
  int data_port = SENTRY_DATA_PORT;
  int tsock_ctrlfd, tsock_datafd; /*  data/ctrl socket */
  int qlen=QLEN;
  int c;
  extern void usage(), finish_sentryd();

#ifdef DEBUG
  debug = 11;
#endif

  openlog("sentryd", LOG_PID, LOG_DAEMON);
  syslog(LOG_NOTICE, "sentryd start");

  if (verbose) {
#ifdef LOGTIME
    logtime = time(NULL);
    log_tm = localtime(&logtime);
    fprintf(stderr, "[%02d/%02d/%02d %02d:%02d:%02d] ",
	    log_tm->tm_year, log_tm->tm_mon+1, log_tm->tm_mday,
	    log_tm->tm_hour, log_tm->tm_min, log_tm->tm_sec);
#endif
    fprintf(stderr, "sentryd start\n");
  }

  signal(SIGQUIT, finish_sentryd);
  signal(SIGTERM, finish_sentryd);
  signal(SIGINT, finish_sentryd);

  sentry_max_datalen = SENTRY_MAX_DATALEN;

  while ((c = getopt(argc, argv, "hDvjp:")) != -1) {
    switch(c) {
    case 'h':
      usage();
      exit (1); /* not reached */
    case 'D':
      debug = 1;
      break;
    case 'v':
      verbose = 1;
      break;
    case 'j':
      require_encode_type = ENCODE_JPEG;
      break;
    case 'p':
      data_port = atoi(optarg);
      ctrl_port = data_port + 1;
      break;
    }
  }

#if 0
  if (argc> 1) {
    data_port = atoi(argv[argc-1]);
    ctrl_port = data_port + 1;
  }
#endif


  /*
   * setup
   */
  if (check_video(NULL)) {
    fprintf(stderr, "sentryd: sorry, video not found.\n");
    exit (2);
  }

  tsock_ctrlfd = build_tcp_port(ctrl_port, qlen);
  tsock_datafd = build_tcp_port(data_port, qlen);
  if (tsock_ctrlfd < 0 || tsock_datafd < 0)
    exit (1);

  (void)mainloop(tsock_ctrlfd, tsock_datafd);

  exit (0);
}


void
usage(void)
{
  printf("usage: sentryd [options]\n\n");

  printf("option:\n");
  printf("       -h       help\n");
  printf("       -j       enable jpeg encode(deafult=disable)\n");
  printf("       -p port  data port(default=%d)\n", SENTRY_DATA_PORT);
  printf("       -D       debug mode\n");
  printf("       -v       verbose mode\n");
  printf("\n");

  exit (1);
}








