/*
 * Sentry video system
 *   is controlled qos according to client's requests.
 *
 *        original by akira-y@is.aist-nara.ac.jp, 1994, 1995
 *        modified by k-chinen@is.aist-nara.ac.jp, 1996
 *        modified by satosi-f@is.aist-nara.ac.jp, 1997
 *
 *  $Id: manage_client.c,v 1.1 1997/08/01 20:30:18 satosi-f Exp satosi-f $
 */

/*
 * manage_client.c - manage (register, release, etc) clients.
 */

#include <stdio.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <errno.h>

#include "sentry.h"
#include "sentry_param.h"

#define MAX_CLIENTS     64
struct {
  u_int flag;                  /* flag (0: invalid, 1: ctrl, 2: data) */
  int sockfd;                  /* socket fd */
  struct sockaddr_in addr;     /* family/address/port */
  union {
    int datafd;     /* datafd  */
    int nctrlfds;   /* #ctrlfd */
    int dummy;      /* dummy   */
  } link;
  int rate;                    /* request rate (only for datafd) */
} client[MAX_CLIENTS];

struct {
  int rate_interval;
  int rate_count;
} client_rate[MAX_CLIENTS];

#if 0
/*
 * get_next_client - return the next valid client_id 
 */
int
get_next_client(int client_id)
{

  while (client_id++ < MAX_CLIENTS) {
    if (client[client_id].flag & (CTRL_SOCKET | DATA_SOCKET))
      return client_id;
  }
  return -1;
}
#endif


u_int
get_client_flag(int client_id)
{
  if (client_id >= 0 && client_id < MAX_CLIENTS)
    return client[client_id].flag;
  return (u_int)-1;
}

int
get_client_sockfd(int client_id)
{
  if (client_id >= 0 && client_id < MAX_CLIENTS)
    return client[client_id].sockfd;
  return -1;
}

struct sockaddr_in *
get_client_sockaddr(int client_id)
{
  if (client_id >= 0 && client_id < MAX_CLIENTS) {
    return &client[client_id].addr;
  }
  return NULL;
}

int
get_client_link_datafd(int client_id)
{
  if (debug)
    fprintf(stderr, "get_client_link_datafd: cid=%d\n", client_id);

  if (client_id >= 0 && client_id < MAX_CLIENTS)
    return client[client_id].link.datafd;
  return -1;
}

int
get_client_link_nctrlfds(int client_id)
{
  if (debug)
    fprintf(stderr, "get_client_link_nctrlfds: cid=%d\n", client_id);

  if (client_id >= 0 && client_id < MAX_CLIENTS)
    return client[client_id].link.nctrlfds;
  return 0;
}

void
set_client_link_datafd(int client_id, int datafd)
{
  if (debug)
    fprintf(stderr, "set_client_link_datafd: cid=%d/ datafd=%d\n",
            client_id, datafd);

  if (client_id >= 0 && client_id < MAX_CLIENTS) {
    if (client[client_id].flag == CTRL_SOCKET)
      client[client_id].link.datafd = datafd;
  }
}

void
set_client_link_nctrlfds(int client_id)
{
  if (debug)
    fprintf(stderr, "set_client_link_nctrlfds: cid=%d\n", client_id);

  if (client_id >= 0 && client_id < MAX_CLIENTS) {
    if (client[client_id].flag & DATA_SOCKET) {
      if (client[client_id].link.nctrlfds < 1)
        client[client_id].link.nctrlfds = 1;
      else
        client[client_id].link.nctrlfds ++;
    }
  }
}

#if 0
#ifdef SUPPORT_MULTIRATE
int
get_client_timer(int client_id, strutct itimerval *val)
{
  if (client >= 0 && client_id < MAX_CLIENTS) {
    if (client[client_id].flag & DATA_SOCKET) {
      val = &(client[client_id].send_timer);
      if (debug) {
	fprintf(stderr, "get_client_timer: interval=%d.%06d/ current=%d.%06d\n",
		val->it_interval.tv_sec, val->it_interval.tv_usec,
		val->it_value.tv_sec, val->it_value.tv_usec);
      }
      return 0;
    }
  }
  val = NULL;
  return -1;
}

int
set_client_timer(int client_id, struct itimerval *val)
{
  if (client_id >= 0 && client_id < MAX_CLIENTS) {
    if (client[client_id].flag & DATA_SOCKET) {
      bcopy((char *)val, (char *)&(client[client_id].send_timer), 
	    sizeof(struct itimerval));
      if (debug) {
	fprintf(stderr, "set_client_timer: client_id=%d [interval=%d.%06d/ current=%d.%06d]\n",
		client_id, 
		client[client_id].send_timer.it_interval.tv_sec,
		client[client_id].send_timer.it_interval.tv_usec,
		client[client_id].send_timer.it_value.tv_sec,
		client[client_id].send_timer.it_value.tv_usec);
      }
      return 0;
    }
  }
  if (debug)
    fprintf(stderr, "set_client_timer: cannot set timer (cid=%d)\n", cid);
  return -1;
}
#endif
#endif

int
get_client_rate(int client_id)
{
  if (client_id >= 0 && client_id < MAX_CLIENTS) {
    if (client[client_id].flag & DATA_SOCKET) {
      return client[client_id].rate;
    }
  }
  return -1;
}

#ifdef SUPPORT_MULTIRATE
int
check_rate_count(int client_id)
{

  if (client_id >= 0 && client_id < MAX_CLIENTS) {
    if (client[client_id].flag & DATA_SOCKET) {
      if (++(client_rate[client_id].rate_count) 
	  >= client_rate[client_id].rate_interval) {

	if (debug > 10) 
	  fprintf(stderr, "check_rate_count: client %d (interval %d) (# %d)\n",
		  client_id, 
		  client_rate[client_id].rate_interval,
		  client_rate[client_id].rate_count);

	client_rate[client_id].rate_count = 0;
	return 1;
      }
    }
  }
  return 0;
}

int
approx(int a, int b)
{
  int quotient, surplus;

  if (debug > 100)
    fprintf(stderr, "a=%d, b=%d\n", a, b);

  if (a <= 0 || b <= 0) 
    return -1;

  quotient = a / b;
  surplus = a % b;
#if 0
  if (surplus * 2 < b)
    return quotient;
  else
    return quotient+1;
#endif
  if (debug)
    fprintf(stderr, "approx: %d / %d = %d\n", a, b, quotient);
  return quotient;
}

void
set_rate_interval(void)
{
  int client_id;
  extern int get_qos_val();

  for(client_id=0; client_id<MAX_CLIENTS; client_id++) {
    if (client[client_id].flag & DATA_SOCKET) {
      client_rate[client_id].rate_interval 
	= approx(get_qos_val(QOS_RATE), client[client_id].rate);

      if (1)
	fprintf(stderr, "client_rate[%d].rate_interval= %d\n", 
		client_id, client_rate[client_id].rate_interval);
    }

  }
}

void
set_client_rate(int client_id, int req_rate)
{
  extern int get_qos_val();

  if (client_id >= 0 && client_id < MAX_CLIENTS) {
    client[client_id].rate = req_rate;
    client_rate[client_id].rate_interval 
	= approx(get_qos_val(QOS_RATE), client[client_id].rate);
    client_rate[client_id].rate_count = 0;
  }

  if (1)
    fprintf(stderr, "set_client_rate: client[%d] (rate %d) (int %d) (# %d)\n",
	    client_id, client[client_id].rate,
	    client_rate[client_id].rate_interval,
	    client_rate[client_id].rate_count);
}
#endif /* SUPPORT_MULTIRATE */

#if 0
get_client_linkid(int client_id)
{
  if (client_id >= 0 && client_id < MAX_CLIENTS)
    return client[client_id].linkid;
  return -1;
}

int
get_client_nlink(int client_id)
{
  if (client_id >= 0 && client_id < MAX_CLIENTS)
    return client[client_id].nlink;
  return 0;
}
#endif


int
find_client_by_fd(int sockfd)
{
  int client_id;

  for(client_id=0; client_id<MAX_CLIENTS; client_id++) {
    if (client[client_id].flag != INVALID &&
        client[client_id].sockfd == sockfd)
      return client_id;
  }
  return -1;
}

int
find_client_by_addr(struct sockaddr_in *cli)
{
  int client_id;

  for(client_id=0; client_id<MAX_CLIENTS; client_id++) {
    if (client[client_id].flag != INVALID &&
        bcmp((char *)&client[client_id].addr, (char *)cli,
             sizeof(struct sockaddr_in)) == 0)
      return client_id;
  }
  return -1;
}

#ifdef OLD
int
find_client_by_addr(u_long addr)
{
  int client_id;

  for(client_id=0; client_id<MAX_CLIENTS; client_id++) {
    if (client[client_id].flag != INVALID &&
        client[client_id].addr.sin_addr.s_addr == addr)
      return client_id;
  }
  return -1;
}
#endif

int
add_client(u_int flag, int sockfd, struct sockaddr_in *addr)
{
  int client_id;

  /* client address check */
  if (addr->sin_family != AF_INET) {
    fprintf(stderr, "sentryd: client address error.\n");
    return -1;
  }

  for(client_id=0; client_id<MAX_CLIENTS; client_id++) {
    if (client[client_id].flag == INVALID) {
      client[client_id].flag = flag;
      client[client_id].sockfd = sockfd;
      client[client_id].link.dummy = -1;
      bcopy((char *)addr, (char *)&client[client_id].addr,
            sizeof(struct sockaddr_in));

      if (1)
        fprintf(stderr, "add_client: id=%d [%s/%d/fd=%d] dummy=%d\n",
                client_id,
                inet_ntoa(ntohl(client[client_id].addr.sin_addr.s_addr)),
                ntohs(client[client_id].addr.sin_port),
                client[client_id].sockfd,
                client[client_id].link.dummy);

      return client_id;
    }
  }

  fprintf(stderr, "sentryd: cannot regist more clients.\n");
  return -1;
}

void
clear_client(int client_id)
{
#ifdef SUPPORT_MULTIRATE
  int cid, maxcid = -1;
  int maxrate = 0;
  extern void set_qos(), set_client_rate();
#endif

  /* client valid flag off */
  if (client_id >= 0 && client_id <= MAX_CLIENTS) {
    if (client[client_id].flag == DATA_SOCKET) {
      client[client_id].link.nctrlfds --;
      if (client[client_id].link.nctrlfds > 0)
        return;
    }
    client[client_id].flag = INVALID;
#ifdef SUPPORT_MULTIRATE
    for(cid=0; cid<MAX_CLIENTS; cid++) {
      if (client[cid].flag == DATA_SOCKET && cid != client_id) {
	if (maxrate < client[cid].rate) {
	  maxrate = client[cid].rate;
	  maxcid = cid;
	}
      }
    }
    if (maxcid >= 0 && maxrate >= 0) {
      if (debug > 100)
        fprintf(stderr, "clear_client: maxcid=%d maxrate=%d\n", maxcid, maxrate);

      set_client_rate(maxcid, maxrate);
      set_qos(QOS_RATE, maxrate);
      set_rate_interval();
      clear_qos_change(QOS_RATE);
    }
#endif /* SUPPORT_MULTIRATE */

    if (debug > 10)
      fprintf(stderr, "clear_client: client[%d] is clear.\n", client_id);
  }
}

#ifdef OLD
void
link_client_fd(int sockfd1, int sockfd2)
{
  int client_id1 = find_client_by_fd(sockfd1);
  int client_id2 = find_client_by_fd(sockfd2);

  if (client_id1 >= 0 && client_id2 >= 0) {
    client[client_id1].linkid = client_id2;
    client[client_id1].nlink ++;
  }
}
#endif


void
init_client(void)
{
  int client_id;

  for(client_id=0; client_id<MAX_CLIENTS; client_id++)
    (void)clear_client(client_id);
}
