/*
 * Sentry video system
 *   is controlled qos according to client's requests.
 *
 *        original by akira-y@is.aist-nara.ac.jp, 1994, 1995
 *        modified by k-chinen@is.aist-nara.ac.jp, 1996
 *        modified by satosi-f@is.aist-nara.ac.jp, 1997
 *
 *  $Id: send_image.c,v 1.1 1997/08/01 20:30:18 satosi-f Exp satosi-f $
 */

/*
 * send_iamge.c - video (chunk of image) handling procedures.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/uio.h>
#include <sys/time.h>
#include <vl/vl.h>
#include <errno.h>

#include "sentry.h"
#include "sentry_param.h"

extern VLServer         svr;
extern VLPath           path;
extern VLNode           src, drn;
extern VLControlValue   val;
extern VLBuffer         buffer;
extern VLInfoPtr        info;

extern char             *dataPtr;
extern int              xsize, ysize;
extern int		grab_xsize, grab_ysize, xoff, yoff;
extern int		frameSize;


int     sentry_max_datalen=SENTRY_MAX_DATALEN;
#if 1
int	require_encode_type=ENCODE_JPEG;
#else
int	require_encode_type=ENCODE_NONE;
#endif

char   *encodePtr;

void
send_image(void)
{
  extern void rate_set();
  /*
  extern void set_para();
  */
  sentry_image_header *hdr;
  int datalen, rest, send_datalen, send_count;
  int new_xsize, new_ysize;
  int nwrite;
  char *sendptr;
  struct timeval tv, waittime;
  extern struct sockaddr_in client_addr;
  int fd;
  extern int maxwritefds;
  extern fd_set writemask;
  fd_set writefds;
#ifdef USE_IOVEC
  /*
  struct msghdr msg;
  */
  sentry_image_header header;
  struct iovec iov[2];
#else
  char packet[MAX_DGRAM_LEN];
#endif /* USE_IOVEC */

#ifdef ALIVE
  extern void set_alive_mark();
#endif /* ALIVE */

  extern int check_qos_change();
  extern void clear_qos_change();
  extern int grab_image(), encode_jpeg(), encode();

  int encode_type;

  if (check_qos_change(QOS_ZOOM) || check_qos_change(QOS_SIZE) ||
      check_qos_change(QOS_COLOR)) {
    clear_qos_change(QOS_ZOOM);
    clear_qos_change(QOS_SIZE);
    clear_qos_change(QOS_COLOR);

    (void)change_para();
  }

  if (debug > 10)
    fprintf(stderr, "send_image: start sending ... \n");

  do {
    /* signap is dependent of SGI machine */
    sginap(1);             /* wait a tick */

    info = vlGetNextValid(svr, buffer);
    /*
    if (debug > 10)
      fprintf(stderr, "send_image: info=%d\n", info);
      */
  } while (!info);

  /* Get a pointer to the frame */
  dataPtr = vlGetActiveRegion(svr, buffer, info);

  gettimeofday(&tv, NULL);

#ifdef ALIVE
  (void)set_alive_mark();
#endif /* ALIVE */

  /*
   * image encode  
   */
  if (debug > 10)
    fprintf(stderr, "send_image: encode type %d frameSize=%d(%dx%d)\n",
            require_encode_type, frameSize, xsize, ysize);

  if (xoff > 0 || yoff > 0) {
    if (debug > 10) 
      fprintf(stderr, "send_image: grab %dx%d+%d+%d\n",
	      grab_xsize, grab_ysize, xoff, yoff);
    if (grab_image(&dataPtr, &frameSize, &xsize, &ysize, grab_xsize, grab_ysize, xoff, yoff) < 0) {
      fprintf(stderr, "send_image: cannot grab image (%dx%d+%d+%d).\n",
	      grab_xsize, grab_ysize,  xoff, yoff);
    }
  }
  if (debug > 10)
    fprintf(stderr, "send_image: dataPtr=%d size=%d (%d x %d)\n",
	    dataPtr, frameSize, xsize, ysize);

  switch(require_encode_type) {
  case ENCODE_JPEG:
    if (debug > 10) 
      fprintf(stderr, "send_image: JPEG_ENCODE .... \n");
    datalen = encode_jpeg(dataPtr, frameSize, xsize, ysize, &new_xsize, &new_ysize, &encodePtr);
    encode_type = ENCODE_JPEG;
    break;
  case ENCODE_NONE:
  dafault:
    if (debug > 10) 
      fprintf(stderr, "send_image: NONE_ENCODE .... \n");
    datalen = encode(dataPtr, frameSize, xsize, ysize, &new_xsize, &new_ysize, &encodePtr);
    encode_type = ENCODE_NONE;
    break;
  }

  if (debug > 10)
    fprintf(stderr, "send_image: type=%d/ datalen=%d/ dataptr=%p/ encodePtr=%p\n",
            encode_type, datalen, dataPtr, encodePtr);



  /* send image */
#ifdef USE_IOVEC
  hdr = &header;

  hdr->hd_ver = SENTRY_VERSION;
  hdr->hd_ts = (htons(tv.tv_sec << 16) | htons((tv.tv_usec >> 4) & 0xffff));

  if (debug > 10) {
#if 0
    fprintf(stderr, "send_image: hd_ts=%u/[sec=%08x/usec=%08x/ts=%08x]\n",
            hdr->hd_ts, tv.tv_sec, tv.tv_usec, hdr->hd_ts);
#else
    fprintf(stderr, "send_image: hd_ts=%x/[sec,usec,ts %010d.%06d,%08x]\n",
            hdr->hd_ts, tv.tv_sec, tv.tv_usec, hdr->hd_ts);
#endif
  }

  hdr->hd_off = 0;
  hdr->width = new_xsize;
  hdr->height = new_ysize;
  hdr->size = datalen;
  hdr->format = get_qos_val(QOS_COLOR);
  hdr->encode = encode_type;


  /**
   ** iov
   **   #0  header
   **   #1  body
   **/

  /*
  bzero((char *)&msg, sizeof(msg));
  msg.msg_name = (caddr_t)&client_addr;
  msg.msg_namelen = sizeof(client_addr);
  */
  iov[0].iov_base = (char *)hdr;
  iov[0].iov_len = sizeof(header);
  /*
  msg.msg_iov = iov;
  msg.msg_iovlen = 2;
  */

  /* wait a tick. I don't know why */
#ifdef USE_WAITTIME
  waittime.tv_sec = 0;
  waittime.tv_usec = 30000;   /* 30 msec */
#endif /* USE_WAITTIME */



  /* write mask check */
  for(fd=0; fd<=maxwritefds; fd++) {
    if (FD_ISSET(fd, &writemask)) {
      if (debug > 100) {
        fprintf(stderr, "send_image: writev: fd# %d, size %8dbytes %4dK\n",
            fd, datalen, TOK(datalen));
      }
    }
    else {
      continue;
    }
#ifdef SUPPORT_MULTIRATE
    if (check_rate_count(find_client_by_fd(fd)) == 0) {
      if (debug) {
	static int nskip = 0;
	if (++nskip >= 100) {
	  fprintf(stderr, "send_image: send skip(%d) (fd %d)\n", nskip, fd);
	  nskip = 0;
	}
      }
      continue;
    }
#endif

    hdr->hd_off = 0;
    rest = datalen;

    sendptr = encodePtr;
    send_count = 0;

    while (rest> 0) {
      send_count ++;

      iov[1].iov_base = sendptr;
      if (rest >= sentry_max_datalen)
        iov[1].iov_len = sentry_max_datalen;
      else
        iov[1].iov_len = rest;

try_write:;
      nwrite = writev(fd, iov, 2);
      /*
      nwrite = sendmsg(fd, &msg, 0);
      */
      if (nwrite < 0) {
        fprintf(stderr, "send_image: writev: errno=%d\n", errno);

#ifdef USE_WAITTIME
        select(0, (fd_set *)0, (fd_set *)0, (fd_set *)0, &waittime);
#else
        perror("send_image: writev");
#endif
        /*
        perror("send_image: sendmsg");
        */
        if(errno==EINTR)
            goto try_write;

        goto fail;
      }

      if (debug > 10) {
        fprintf(stderr,
            "send_image: %d# %4d, %8d bytes %4dK (offset %8d)\n",
                  fd, send_count, nwrite, TOK(nwrite), hdr->hd_off);
      }

      send_datalen = nwrite - sizeof(header);
      hdr->hd_off += send_datalen;
      sendptr += send_datalen;
      rest -= send_datalen;

    } /* while for write-loop */

fail:;

  } /* for */

#else
  /* !USE_IOVEC */
  fprintf(stderr, "Now, only USE_IOVEC version is supported.\n");
  exit (1);
#endif /* USE_IOVEC */

  /*
   * Finished with frame, unlocked the buffer
   */
  vlPutFree(svr, buffer);

  /*
   * rate set
   */
  rate_set();
}

/*
 * grab image (now, only supported for ysize offset)
 */
int
grab_image(char **data, int* size, int* x, int* y, int gx, int gy, int xoff, int yoff)
{
  int color_depth;
  extern int get_qos_val();

  if (debug > 10)
    fprintf(stderr, "grab_image: original image imagePtr=%u size=%d (%d x %d)\n",
	    *data, *size, *x, *y);
  switch(get_qos_val(QOS_COLOR)) {
  case FULL_COLOR:
    color_depth = 4;
    break;
  case MONOCHROME:
  default:
    color_depth = 1;
    break;
  }

  *data = &(*data)[(*x)*yoff*color_depth];
  (*y) = gy;
  (*size) = gx * gy * color_depth;

  if (debug > 10)
    fprintf(stderr, "grab_image: newimage imagePtr=%u size=%d (%d x %d)\n",
	    *data, *size, *x, *y);

  return 0;
}


  
#ifdef ALIVE
/*
 * plot alive mark in top line
 */
void
set_alive_mark(void)
{
  int i, color_depth;
  static int alive_pos = 0;
  extern int get_qos_val();
#if 0
  int j,debugpoint;
#endif

  switch (get_qos_val(QOS_COLOR)) {
  case FULL_COLOR:
    color_depth = 4;
    break;
  case MONOCHROME:
  default:
    color_depth = 1;
    break;
  }

  for(i=0; i<color_depth*2; i++) {
    dataPtr[alive_pos+i] = 0xff;
  }
#if 0
  debugpoint = 10 * 160 * color_depth;
  for(i=10; i<20; i++) {
    for(j=0; j<color_depth*2; j++) {
      dataPtr[debugpoint+j] = 0xff;
    }
    debugpoint += 160 * color_depth;
  }
#if 0
  debugpoint = 100 * 160 * color_depth;
  for(i=100; i<110; i++) {
    for(j=0; j<color_depth*2; j++) {
      dataPtr[debugpoint+j] = 0xff;
    }
    debugpoint += 160 * color_depth;
  }
#endif
#endif
  if (alive_pos + i - 1 < xsize*color_depth)
    alive_pos += i;
  else
    alive_pos = 0;
}
#endif /* ALIVE */


