/*
 * Sentry video system
 *   is controlled qos according to client's requests.
 *
 *        original by akira-y@is.aist-nara.ac.jp, 1994, 1995
 *        modified by k-chinen@is.aist-nara.ac.jp, 1996
 *        modified by satosi-f@is.aist-nara.ac.jp, 1997
 *
 *  $Id: video.c,v 1.1 1997/08/01 20:30:18 satosi-f Exp satosi-f $
 */

/*
 * video.c - video (chunk of images) handling procedures
 */

#include <stdio.h>
#include <sys/time.h>
#include <vl/vl.h>
#include <errno.h>

#include "sentry.h"
#include "sentry_param.h"


extern int debug;
extern int errno;


VLServer        svr;
VLPath          path;
VLNode          src, drn;
VLControlValue  val;
VLBuffer        buffer;
VLInfoPtr       info;

char            *dataPtr;
int             xsize, ysize;
int		grab_xsize, grab_ysize, xoff, yoff;
int             frameSize;


void
error_exit(void)
{
  vlPerror("open video");
  exit (1);
}

int
check_video(char *vserver)
{
  int ret = 0;

  if (debug > 100)
    fprintf(stderr, "check_video ... ");

  /* check if vl is running */
  if ((svr = vlOpenVideo(vserver)) == NULL) {
    ret = 1;
  }
  vlCloseVideo(svr);
  if (debug > 10)
    fprintf(stderr, "check_video: rev=%d\n", ret);

  return ret;
}

void
init_video(char *vserver)
{
  extern void setup_video();

  /* Connect to the daemon */
  if (debug > 100)
    fprintf(stderr, "vlOpenVideo ... ");
  if ((svr = vlOpenVideo(vserver)) == NULL)
    error_exit();
  if (debug > 100)
    fprintf(stderr, "ok\n");

  /* Set up a drain node in memory */
  if (debug > 100)
    fprintf(stderr, "vlGetNode (drn) ... ");
  drn = vlGetNode(svr, VL_DRN, VL_MEM, VL_ANY);
  if (debug > 100)
    fprintf(stderr, "ok\n");

  /* Set up a source node on any video sources */
  if (debug > 100)
    fprintf(stderr, "vlGetNode (src) ... ");
  src = vlGetNode(svr, VL_SRC, VL_VIDEO, VL_ANY);
  if (debug > 100)
    fprintf(stderr, "ok\n");

  (void)setup_video();
}

void
setup_video(void)
{
  extern void set_color(), set_zoom(), set_size();

  /* Create a path using the first device that will supprot it */
  if (debug > 100)
    fprintf(stderr, "vlCreatePath ... ");
  path = vlCreatePath(svr, VL_ANY, src, drn);
  if (debug > 100)
    fprintf(stderr, "ok\n");

  /* Set up the hardware for and define the usage of the path */
  if (debug > 100)
    fprintf(stderr, "vlSetupPaths ... ");
  if ((vlSetupPaths(svr, (VLPathList)&path, 1, VL_SHARE, VL_SHARE)) < 0)
    error_exit();
  if (debug > 100)
    fprintf(stderr, "ok\n");

  (void)set_zoom();
  (void)set_size();
  (void)set_color();

  frameSize = vlGetTransferSize(svr, path);
  if (debug)
    fprintf(stderr, "setup_video: frameSize=%d\n", frameSize);

  /* Get the video size */
  vlGetControl(svr, path, drn, VL_SIZE, &val);
  xsize = val.xyVal.x;
  ysize = val.xyVal.y;
  printf("open_video: frameSize = %d (%d x %d)\n",
	 frameSize, xsize, ysize);

  /* Create and register a buffer for 1 frame */
  buffer = vlCreateBuffer(svr, path, drn, 1);
  if (buffer == NULL)
    error_exit();
  vlRegisterBuffer(svr, path, drn, buffer);

  /* Begin the data transfer */
  if (vlBeginTransfer(svr, path, 0, NULL))
    error_exit();
}

void
set_color(void)
{
  extern int get_qos_val();

  /* Set the packing to RGB */
  switch (get_qos_val(QOS_COLOR)) {
  case FULL_COLOR:
    val.intVal = VL_PACKING_RGB_8;
    break;
  case MONOCHROME:
  default:
    val.intVal = VL_PACKING_Y_8_P;
    break;
  }
  if (debug)
    fprintf(stderr, "set_color: val.intVal=%d\n",
            val.intVal);

  if (vlSetControl(svr, path, drn, VL_PACKING, &val) < 0) {
    vlPerror("VL_PACKING");
    /*
    error_exit();
    */
  }
}

void
set_size(void)
{
  extern int get_qos_val();
  VLControlValue offset,offset2;

  offset.xyVal.x = offset.xyVal.y = 0;
  offset2.xyVal.x = offset2.xyVal.y = 0;
  xoff = yoff = 0;

  /* Set the frame size */
  switch (get_qos_val(QOS_SIZE)) {
  case SIZE_640x480:
    val.xyVal.x = 640;
    val.xyVal.y = 480;
    break;
  case SIZE_640x360:
    val.xyVal.x = 640;
    val.xyVal.y = 360;
    offset.xyVal.x = 0;
    offset.xyVal.y = 60;
    offset2.xyVal.y = 60;
    break;
  case SIZE_320x240:
    val.xyVal.x = 320;
    val.xyVal.y = 240;
    break;
  case SIZE_320x180:
    val.xyVal.x = 320;
    val.xyVal.y = 180;
    offset.xyVal.x = 0;
    offset.xyVal.y = 30;
    offset2.xyVal.y = 30;
    break;
  case SIZE_160x120:
    val.xyVal.x = 160;
    val.xyVal.y = 120;
    break;
  case SIZE_160x90:
    val.xyVal.x = 160;
    val.xyVal.y = 90;
    offset.xyVal.x = 0;
    offset.xyVal.y = 14;
    offset2.xyVal.y = 16;
    break;
  case SIZE_212x160:
  default:
    val.xyVal.x = 212;
    val.xyVal.y = 160;
    break;
  case SIZE_212x120:
    val.xyVal.x = 212;
    val.xyVal.y = 120;
    offset.xyVal.x = 0;
    offset.xyVal.y = 20;
    offset2.xyVal.y = 20;
    break;
  }
  
  if (debug)
    fprintf(stderr, "set_size: size/offset (%dx%d+%d+%d)\n",
	    val.xyVal.x, val.xyVal.y, offset.xyVal.x, offset.xyVal.y);

  if (vlSetControl(svr, path, drn, VL_OFFSET, &offset) < 0) {
    vlPerror("VL_OFFSET");
    xoff = offset.xyVal.x;
    yoff = offset.xyVal.y;
    grab_xsize = val.xyVal.x;
    grab_ysize = val.xyVal.y;
    val.xyVal.x += xoff + offset2.xyVal.x;
    val.xyVal.y += yoff + offset2.xyVal.y;
  }

  if (vlSetControl(svr, path, drn, VL_SIZE, &val) < 0) {
    vlPerror("VL_SIZE");
    /*
      error_exit();
      */
  }

}

void
set_zoom(void)
{
  extern int get_qos_val();

  /* Set the zoom ratio */
  val.fractVal.numerator = 1;
  val.fractVal.denominator = get_qos_val(QOS_ZOOM);

  if (debug)
    fprintf(stderr, "set_zoom: val.fractVal.numerator=%d/ val.fractVal.denominator = %d\n",
            val.fractVal.numerator, val.fractVal.denominator);

  if (vlSetControl(svr, path, drn, VL_ZOOM, &val) < 0) {
    vlPerror("VL_ZOOM");
    /*
    error_exit();
    */
  }
}

void
change_para(void)
{
  extern void setup_video();

  if (debug)
    fprintf(stderr, "change_para ... start.\n");

  vlEndTransfer(svr, path);
  vlDeregisterBuffer(svr, path, drn, buffer);
  vlDestroyBuffer(svr, buffer);
  vlDestroyPath(svr, path);

  (void)setup_video();

  if (debug)
    fprintf(stderr, "change_para ... ok\n");
}

